#ifndef _camera_h
#define _camera_h

/*************************************************************************
 *  DccNiTghtmare is public domain. Do whatever you want with this code. *
 *************************************************************************/

#include <SDL/SDL.h>
#include <SDL/SDL_opengl.h>

#include "../etc/util.h"

#define CAMERA_TYPE_NORMAL 1  /**< Normal Camera Mode */
#define CAMERA_TYPE_DRIVE  2  /**< Drive Camera Mode */

/*! The Camera Manipulation Class */
class camera
{
   public:
      /*! Constructor */
      camera();
      /*! Destructor */
      ~camera();

      /*! Do the IO of the Camera 
       * \param keys -> pointer to the SDL Keys
       * \param mBotao -> mouse Button state
       * \param x -> mouse X Screen position
       * \param y -> mouse Y Screen position
       * \param varCamera -> how much the camera moves
       * \return TRUE if camera modified, false otherwise */
      bool doIO(Uint8 *keys, Uint8 mBotao, int x, int y, GLfloat varCamera);
   
      /*! Set the state of camera move with mouse.
       * \param value -> true will enable change the camera X/Z
       *                 position with left mouse button. false will
       *                 disable it. */
      void setMoveWithMouse(bool value);

      /*! Get the state of camera move with mouse.
       * \return -> true if is enabled the change of camera X/Z
       *            position with left mouse button. Or false if disabled. */
      bool getMoveWithMouse();

      /*! Get The Camera X Postion
       * \return Camera X position */
      GLfloat getCameraX();
      /*! Get The Camera Y Postion
       * \return Camera Y position */
      GLfloat getCameraY();
      /*! Get The Camera Z Postion
       * \return Camera Z position */
      GLfloat getCameraZ();

      /*! Sum Value to Current Camera Theta
       * \param f -> value to sum */
      void sumTheta(GLfloat f);
      /*! Sum Value to Current Camera Phi
       * \param f -> value to sum */
      void sumPhi(GLfloat f);
      /*! Set the camera Phi, doing the acceleration
       * \param f -> new camera phi value */
      void setPhi(GLfloat f);
      /*! Sum Value to Current Camera D (Zoom)
       * \param f -> value to sum */
      void sumD(GLfloat f);

      /*! Get The Camera Phi
       * \return Camera Phi */
      GLfloat getPhi();
      /*! Get The Camera Theta
       * \return Camera Theta */
      GLfloat getTheta();
      /*! Get The Camera DeltaY
       * \return Camera DeltaY */
      GLfloat getDeltaY();
      /*! Get The Camera D (Zoom)
       * \return Camera D (Zoom) */
      GLfloat getD();
      
      /*! Get The Camera Center X Postion
       * \return Camera Center X position */
      GLfloat getCenterX();
      /*! Get The Camera Center Y Postion
       * \return Camera Center Y position */
      GLfloat getCenterY();
      /*! Get The Camera Center Z Postion
       * \return Camera Center Z position */
      GLfloat getCenterZ();

      /*! Do the camera Look At */
      void lookAt();

      /*! Define new Camera Type
       * \param value -> value of the new camera type */
      void defineCameraType(int value);

      /*! Update Camera Position
       * \param characterX -> character X position 
       * \param characterY -> character Y position
       * \param characterZ -> character Z position
       * \param characterOrientation -> character Orientation */
      void updateCamera(GLfloat characterX, GLfloat characterY, 
                        GLfloat characterZ, GLfloat characterOrientation);

      /*! Change the minimun zoom value. Usefull for other applications,
       * like the map editor, who needs to be more "far" 
       * \param value -> new value of the minimun zoom */
      void defineMinZoom(GLfloat value);

      /*! Push current camera values to the 1-sized queue */
      void push();

      /*! Retrieve camera values from the 1-sized queue */
      void pop();

   private:
      GLfloat theta;                    /**< Rotation Up*/
      GLfloat phi;                      /**< Rotation to Sides*/
      GLfloat d;                        /**< Zoom */
      GLfloat cameraX,                  /**< Camera X Position */
              cameraY,                  /**< Camera Y Position */
              cameraZ;                  /**< Camera Z Position */
      GLfloat deltaY;                   /**< Camera DeltaY */
      GLfloat centerX,                  /**< Camera Central X Position */
              centerY,                  /**< Camera Central Y Position */
              centerZ;                  /**< Camera Central Z Position */

      GLfloat queue[13];                /**< queue to keep values for pushpop */

      bool middleMouse;                 /**< Middle Mouse Camera Control */  
      int prevX,                        /**< Previous Mouse X position */
          prevY;                        /**< Previous Mouse Z position */

      int type;                         /**< Internal Camera Type */

      GLfloat zoomAc;                   /**< Zoom Acceleration Value */
      GLfloat phiAc;                    /**< Phi Acceleration Value */
      GLfloat tgtPhi;                   /**< Phi Target Value */
      GLfloat thetaAc;                  /**< Theta Acceleration Value */
      GLfloat minZoom;                  /**< The max Zoom Value */

      bool moveWithMouse;        /**< If change the Xz position with mouse */
      int mousePositionX;        /**< Last mouse X position when changed mWM */
      int mousePositionY;        /**< Last mouse X position when changed mWM*/
      int fixedX,                /**< Fixed the X when change the mWM */
          fixedZ;                /**< Fixed the Y when change the mWM */
};

#endif

