/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "fieldpub.h"

#include "../etc/draw.h"
#include "../etc/dirs.h"

#include <SDL/SDL.h>
#include <SDL/SDL_image.h>


////////////////////////////////////////////////////////////////////////
//                                                                    //
//                            publicity                               //
//                                                                    //
////////////////////////////////////////////////////////////////////////

/*************************************************************
 *                         Constructor                       *
 *************************************************************/
publicity::publicity(string textureFile, 
                     float x, float y, float z, float alpha)
{
   dirs dir;

   /* Set Position */
   posX = x;
   posY = y;
   posZ = z;
   angle = alpha;

   /* Load Texture */
   glGenTextures(1, &texture);
   SDL_Surface* img = IMG_Load(dir.getRealFile(textureFile).c_str());
   setTexture(img, texture, true);
   SDL_FreeSurface(img);

   next = NULL;
   previous = NULL;
}

/*************************************************************
 *                          Destructor                       *
 *************************************************************/
publicity::~publicity()
{
   glDeleteTextures(1, &texture);
}

/*************************************************************
 *                          getTexture                       *
 *************************************************************/
GLuint publicity::getTexture()
{
   return(texture);
}

/*************************************************************
 *                         getPosition                       *
 *************************************************************/
void publicity::getPosition(float &x, float &y, float &z, float &alpha)
{
   x = posX;
   y = posY;
   z = posZ;
   alpha = angle;
}

////////////////////////////////////////////////////////////////////////
//                                                                    //
//                          publicityModel                            //
//                                                                    //
////////////////////////////////////////////////////////////////////////

/*************************************************************
 *                         Constructor                       *
 *************************************************************/
publicityModel::publicityModel(string fileName)
{
   /* Load Model */
   modelFile = fileName;
   model = new aniModel();
   model->loadModel(modelFile);
   /* Since there aren't animations, the bounding box is static */
   model->calculateBoundingBox(modelMin, modelMax);

   /* Set The Pblicities List */
   list = NULL;
   total = 0;
}

/*************************************************************
 *                          Destructor                       *
 *************************************************************/
publicityModel::~publicityModel()
{
   /* Delete Model */
   delete(model);

   /* Clear the List */
   int i;
   publicity* tmp;
   for(i = 0; i < total; i++)
   {
      tmp = list;
      list = list->next;
      delete(tmp);
   }
   total = 0;
   list = NULL;
}

/*************************************************************
 *                         getModelFile                      *
 *************************************************************/
string publicityModel::getModelFile()
{
   return(modelFile);
}

/*************************************************************
 *                         addPublicity                      *
 *************************************************************/
void publicityModel::addPublicity(string textureName,
                                  float x, float y, float z, float alpha)
{
   publicity* pub = new publicity(textureName, x, y, z, alpha);

   if(total == 0)
   {
      pub->next = pub;
      pub->previous = pub;
   }
   else
   {
      pub->next = list;
      pub->previous = list->previous;
      pub->next->previous = pub;
      pub->previous->next = pub;
   }
   list = pub;
   total++;
}

/*************************************************************
 *                            render                         *
 *************************************************************/
void publicityModel::render()
{
   publicity* pub = list;
   int i;
   float x=0, y=0, z=0, alpha=0;

   model->loadToGraphicMemory(false);

   for(i = 0; i < total; i++)
   {
      pub->getPosition(x,y,z,alpha);

      glBindTexture(GL_TEXTURE_2D, pub->getTexture());
      glPushMatrix();
         glTranslatef(x, y, z);
         glRotatef(alpha, 0,1,0);
         model->renderFromGraphicMemory();
      glPopMatrix();

      pub = pub->next;
   }

   model->removeFromGraphicMemory();

}

////////////////////////////////////////////////////////////////////////
//                                                                    //
//                            fieldPub                                //
//                                                                    //
////////////////////////////////////////////////////////////////////////

/*************************************************************
 *                         Constructor                       *
 *************************************************************/
fieldPub::fieldPub()
{
   /* Init List */
   list = NULL;
   total = 0;
}

/*************************************************************
 *                          Destructor                       *
 *************************************************************/
fieldPub::~fieldPub()
{
   /* Empty the list */
   publicityModel* pub;
   int i;
   for(i = 0; i < total; i++)
   {
      pub = list;
      list = list->next;
      delete(pub);
   }
}

/*************************************************************
 *                         addPublicity                      *
 *************************************************************/
void fieldPub::addPublicity(string modelName, string textureName,
                            float x, float y, float z, float alpha)
{
   publicityModel* foundModel = NULL;
   publicityModel* model = list;
   int i;
   /* First search for the model */
   for(i = 0; (i < total) && (foundModel == NULL); i++)
   {
      if(model->getModelFile() == modelName)
      {
         foundModel = model;
      }
      model = model->next;
   }

   /* If not found, add it */
   if(!foundModel)
   {
      foundModel = new publicityModel(modelName);
      if(total == 0)
      {
         foundModel->next = foundModel;
         foundModel->previous = foundModel;
      }
      else
      {
         foundModel->next = list;
         foundModel->previous = list->previous;
         foundModel->next->previous = foundModel;
         foundModel->previous->next = foundModel;
      }
      list = foundModel;
      total++;
   }

   /* So, use it! */
   foundModel->addPublicity(textureName, x, y, z, alpha);
}

/*************************************************************
 *                            render                         *
 *************************************************************/
void fieldPub::render()
{
   int i;
   publicityModel* model = list;
   for(i = 0; i < total; i++)
   {
      model->render();
      model = model->next;
   }
}

