/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <SDL/SDL_image.h>

#include "replay.h"

#include "../gui/dntfont.h"

#include "../etc/draw.h"
#include "../etc/options.h"

#define REPLAY_TEXTURE_DELTA  0.05

/**************************************************************
 *                          Constructor                       *
 **************************************************************/
replay::replay(goal *g, camera *cam, field* pfield)
{
   teamA = NULL;
   teamB = NULL;
   gameBall = NULL;
   
   goals = g;
   gameCamera = cam;
   gameField = pfield;
   
   start = 0;
   currentFrame = 0;
   end = 0;
   replaying = false;

   /* Create the Little R Texture */
   defineReplayTexture();

   /* Load the Controller Texture */
   controller = new videoController();
}

/**************************************************************
 *                          Destructor                        *
 **************************************************************/
replay::~replay()
{
   /* Delete the replay texture */
   glDeleteTextures(1, &replayTexture);

   /* Delete the video controller */
   delete(controller);
}

/**************************************************************
 *                     defineReplayTexture                    *
 **************************************************************/
void replay::defineReplayTexture()
{
   /* Define Machine Bit Order */
   Uint32 rmask, gmask, bmask, amask;
   #if SDL_BYTEORDER == SDL_BIG_ENDIAN
      rmask = 0xff000000;
      gmask = 0x00ff0000;
      bmask = 0x0000ff00;
      amask = 0x000000ff;
   #else
      rmask = 0x000000ff;
      gmask = 0x0000ff00;
      bmask = 0x00ff0000;
      amask = 0xff000000;
   #endif

   /* Set the desired font */
   dntFont fnt;
   fnt.defineFont(DNT_FONT_ARIAL, 40);
   fnt.defineFontAlign(DNT_FONT_ALIGN_CENTER);
   fnt.defineFontStyle(DNT_FONT_STYLE_NORMAL);

   /* Create the Surface to write to */
   SDL_Surface* img;
   img = SDL_CreateRGBSurface(SDL_SWSURFACE, 64, 64, 32,
                                    rmask, gmask, bmask, amask);

   /* Write text to Surface */
   color_Set(255, 255, 255, 255);
   fnt.write(img,0, 0, "R", 0, 0, 63, 63, true); 

   /* Set Texture */
   glGenTextures(1, &replayTexture);
   setTextureRGBA(img, replayTexture);

   /* Free the temporary surface */
   SDL_FreeSurface(img);

   replayTextureScale = 1.0;
   replayTextureDelta = -REPLAY_TEXTURE_DELTA;
}


/**************************************************************
 *                           setTeamA                         *
 **************************************************************/
void replay::setTeamA(team *t)
{
   teamA = t;
}

/**************************************************************
 *                           setTeamB                         *
 **************************************************************/
void replay::setTeamB(team *t)
{
   teamB = t;
}

/**************************************************************
 *                           setBall                          *
 **************************************************************/
void replay::setBall(ball *b)
{
   gameBall = b;
}

/**************************************************************
 *                            clear                           *
 **************************************************************/
void replay::clear()
{
   end = start;
}

/**************************************************************
 *                          updateData                        *
 **************************************************************/
void replay::updateData()
{
   if (replaying) 
      return;
      
   /* getting goalKeeper positions and angles*/
   if (teamA)
   {
      data[end].teamAPositions[0][0] = teamA->goalKeeper.getPosX();
      data[end].teamAPositions[0][1] = teamA->goalKeeper.getPosZ();
      data[end].teamAAngles[0] = teamA->goalKeeper.getAngle();
   }
   
   if (teamB)
   {
      data[end].teamBPositions[0][0] = teamB->goalKeeper.getPosX();
      data[end].teamBPositions[0][1] = teamB->goalKeeper.getPosZ();
      data[end].teamBAngles[0] = teamB->goalKeeper.getAngle();
   }
   
   /* getting disk positions and angles*/
   for (int i=0; i<10; i++)
   {
      if (teamA)
      {
         data[end].teamAPositions[i+1][0] = teamA->disk[i].getPosX();
         data[end].teamAPositions[i+1][1] = teamA->disk[i].getPosZ();
         data[end].teamAAngles[i+1] = teamA->disk[i].getAngle();
      }
          
      if (teamB)
      {
         data[end].teamBPositions[i+1][0] = teamB->disk[i].getPosX();
         data[end].teamBPositions[i+1][1] = teamB->disk[i].getPosZ();
         data[end].teamBAngles[i+1] = teamB->disk[i].getAngle();
      }
   }
   
   /* getting ball position and angles */
   if (gameBall)
   {
      data[end].ballPosition[0] = gameBall->getPosX();
      data[end].ballPosition[1] = gameBall->getPosY();
      data[end].ballPosition[2] = gameBall->getPosZ();
            
      data[end].ballAngle[0] = gameBall->getAngleX();
      data[end].ballAngle[1] = gameBall->getAngleY();
      data[end].ballAngle[2] = gameBall->getAngleZ();
   }
   
   /* setting new vector end position */
   end = (end+1) % FRAMES_TO_REPLAY;
   
   /* if vector is full, shift elements */
   if (end == start)
   {
      start = (start + 1) % FRAMES_TO_REPLAY;
   }
   
}

/**************************************************************
 *                           getData                          *
 **************************************************************/   
replayData replay::getData(int frame)
{
   /* transforming frame index from 0-FRAMES_TO_REPLAY to start-end */
   frame = frame+start;
   
   /* making sure frame is inside start-end interval */
   if (end < start)
   {
      int end2 = end+FRAMES_TO_REPLAY;
      if (frame >= end2)
      {
         frame = end2 - 1;
      }
   }
   else if (frame >= end)
   {
      frame = end-1;
   }
   
   frame = frame % FRAMES_TO_REPLAY;
   
   return data[frame];
}

/**************************************************************
 *                    renderReplayTexture                     *
 **************************************************************/
void replay::renderReplayTexture()
{
   options opt;

   /* Update Scale Factor */
   replayTextureScale += replayTextureDelta;
   if(replayTextureScale <= 0.4)
   {
      replayTextureScale = 0.4;
      replayTextureDelta *= -1;
   }
   else if(replayTextureScale >= 1.0)
   {
      replayTextureScale = 1.0;
      replayTextureDelta *= -1;
   }

   /* Set the texture */
   glEnable(GL_TEXTURE_2D);
   glEnable(GL_BLEND);
   glBindTexture(GL_TEXTURE_2D, replayTexture);

   /* Finally, Draw the R */
   glPushMatrix();
      glTranslatef(16, opt.getWindowY()-16, 0);
      glScalef(replayTextureScale, replayTextureScale, 1.0);
      glBegin(GL_QUADS);
         glTexCoord2f(0,1);
         glVertex2f(-16, -16);
         glTexCoord2f(1,1);
         glVertex2f(16, -16);
         glTexCoord2f(1,0);
         glVertex2f(16, 16);
         glTexCoord2f(0,0);
         glVertex2f(-16, 16);
      glEnd();
   glPopMatrix();

   /* Reset things */
   glDisable(GL_TEXTURE_2D);
   glDisable(GL_BLEND);
}

/**************************************************************
 *                           replayIO                         *
 **************************************************************/
void replay::replayIO()
{
   /* Get Keys and Mouse State */
   SDL_PumpEvents();
   keys = SDL_GetKeyState(NULL);
   mButton = SDL_GetMouseState(&mouseX,&mouseY);

   /* Do Camera IO */
   gameCamera->doIO(keys, mButton, mouseX, mouseY, DELTA_CAMERA);

   /* Do the controller IO */
   switch(controller->verifyButtons(mButton, mouseX, mouseY))
   {
      /* "Flux" Controls */
      case VIDEO_CONTROLLER_STOP:
      {
         /* Pause the replay */
         delta = 0;
         /* Put it at initial position */
         position = start;
      }
      break;
      case VIDEO_CONTROLLER_PLAY:
      {
         /* Start the play at current speed */
         delta = curSpeed;
      }
      break;
      case VIDEO_CONTROLLER_PAUSE:
      {
         /* Pause the play */
         delta = 0.0;
      }
      break;

      /* "Direction Controls" */
      case VIDEO_CONTROLLER_BACKWARD:
      {
         /* Backward the play */
         position -= curSpeed;
      }
      break;
      case VIDEO_CONTROLLER_FORWARD:
      {
         /* Forward the play */
         position += curSpeed;
      }
      break;

      /* Velocity Controls */
      case VIDEO_CONTROLLER_HALF_SPEED:
      {
         curSpeed = 0.5;
         if(delta)
         {
            delta = curSpeed;
            position += delta;
         }
      }
      break;
      case VIDEO_CONTROLLER_NORMAL_SPEED:
      {
         curSpeed = 1.0;
         if(delta)
         {
            delta = curSpeed;
            position += delta;
         }
      }
      break;
      case VIDEO_CONTROLLER_DOUBLE_SPEED:
      {
         curSpeed = 2.0;
         if(delta)
         {
            delta = curSpeed;
            position += delta;
         }
      }
      break;

      /* ShutDown Controller  */
      case VIDEO_CONTROLLER_SHUTDOWN:
      {
         exit = true;
      }
      break;

      case VIDEO_CONTROLLER_NOTHING:
      default:
      {  
         /* Update the position */
         position += delta;
      }
      break;

   }

   /* FIXME -> the bellow IO is for debug only and must be removed! */

   /* Do the debug keyboard IO */
   if (keys[SDLK_ESCAPE])
   {
      exit = true;
   }
}

/**************************************************************
 *                         renderReplay                       *
 **************************************************************/
void replay::renderReplay(cursor* cursors)
{
   int frame;
   int end2;
   Uint32 time = 0, lastTime = 0;
   Uint32 varTime = 0;
   exit = false;
   
   if (end < start)
   {
      end2 = end + FRAMES_TO_REPLAY;
   }
   else
   {
      end2 = end;
   }
   
   position = start;
   curSpeed = 1.0;
   delta = curSpeed;
   while (!exit)
   {
      time = SDL_GetTicks();
      varTime = time - lastTime;
      
      if( ((varTime)) >= UPDATE_RATE)
      {
         lastTime = time;
         
         /* Get the current frame to display */
         frame = (int)(position) % FRAMES_TO_REPLAY;
         
         /* Verify the Replay I/O */
         replayIO();
   
         /* Verify Position Limits */
         if (position < start) 
         {
            position = start;
         }
         else if (position >= end2)
         {
            position = end2-1;
         }
         
         /* Render Replay Frame */
         renderFrame(frame, cursors);

      }
      else
      {
         Uint32 sleepTime = (int) ((UPDATE_RATE-1) - varTime);
         if( (sleepTime > 0) && (sleepTime < UPDATE_RATE))
         {
            SDL_Delay(sleepTime);
         }
      }
   }
   
   /* wait untill user releases escape key */
   while (keys[SDLK_ESCAPE])
   {
      SDL_PumpEvents();
      keys = SDL_GetKeyState(NULL);
   }
   
}

/**************************************************************
 *                          renderFrame                       *
 **************************************************************/
void replay::renderFrame(int frame, cursor* cursors)
{
   /* Clear Things */
   glClearColor(0.1,0.1,0.8,1.0);
   glClear (GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
   glLoadIdentity();

   /* Redefine camera position */
   gameCamera->lookAt();

   /* Render the table */
   if(gameField)
   {
      gameField->render();
   }

   /* Render Ball */
   if (gameBall)
   {
      gameBall->render(data[frame].ballPosition, data[frame].ballAngle);
   }

   /* Render Teams */
   if (teamA)
   {
      teamA->render(data[frame].teamAPositions, data[frame].teamAAngles);
   }
   
   if (teamB)
   {
      teamB->render(data[frame].teamBPositions, data[frame].teamBAngles);
   }

   /* Render goals */
   goals->render(gameCamera);

   /* Set 2D Mode */
   draw2DMode();

   /* Disable Things */
   glDisable(GL_FOG);
   glDisable(GL_LIGHTING);

   /* Render the Replay Texture */
   renderReplayTexture();

   /* Render Controller */
   controller->render();

   /* Render Mouse Cursor */
   glPushMatrix();
      cursors->draw(mouseX, mouseY);
   glPopMatrix();

   /* Return to 3D Mode */
   draw3DMode(FARVIEW);

   /* Renable Things */
   glEnable(GL_FOG);
   glEnable(GL_LIGHTING);

   /* Flush to Screen */
   glFlush();
   SDL_GL_SwapBuffers();

}

