#include "xut.h"

#ifdef _MSC_VER
   #include "../config_win.h"
#else
   #include "../config.h"
#endif

#include "../etc/draw.h"
#include "../etc/dirs.h"

#include "../gui/selector.h"
#include "../gui/dntfont.h"

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#define CUBE_SIZE 1000


/********************************************************************
 *                           Constructor                            *
 ********************************************************************/
xut::xut(SDL_Surface* screen, int scWidth, int scHeight)
{
   /* Set window */
   actualScreen = screen;
   screenWidth = scWidth;
   screenHeight = scHeight;

   /* Load Back Texture */
   loadBackgroundTexture();

   /* Create Mouse Cursors */
   cursors = new cursor();
   
   /* Create Rule System */
   rulesSystem = new rules();

   /* Create main gui */
   gameMainGui = new mainGui();
   
   /* Nullify Teams */
   teamA = NULL;
   teamB = NULL;

   /* Create Goals */
   goals = new goal("models/goal/model.cfg");

   /* Create Balls */
   gameBall = new ball();

   /* Set up physics engine */
   physic.setBall(gameBall);
   physic.setGoals(goals);

   rulesSystem->setBall(gameBall);
   
   /* Create Camera */
   gameCamera = new camera();

   /* Create guiMessage */
   gameMessage.init("gui/main/info.png");

   /* Create the Field */
   gameField = new field();
   //TODO -> better do, latter, a stadium selector
   switch(opt.getFieldDetail())
   {
      case XUT_OPTIONS_FIELD_DETAIL_LOW:
         gameField->createField("models/field/botao_campo_512.png");
      break;
      case XUT_OPTIONS_FIELD_DETAIL_NORMAL:
         gameField->createField("models/field/botao_campo.png");
      break;
      case XUT_OPTIONS_FIELD_DETAIL_HIGH:
         gameField->createField("models/field/botao_campo_2048.png");
      break;
   }
 
   /* Create the Replay Controller  */
   replayer = new replay(goals, gameCamera, gameField);
   replayer->setBall(gameBall);
      
   /* Init Variables */
   exit = false;
   selectedPlayer = NULL;
   initedTurn = true;
}

/********************************************************************
 *                           Destructor                             *
 ********************************************************************/
xut::~xut()
{
   glDeleteTextures(1, &bgTexture);
   gameField->deleteField();
   delete(gameMainGui);
   delete(cursors);
   delete(gameField);
   delete(rulesSystem);
   delete(teamA);
   delete(teamB);
   delete(gameBall);
   delete(goals);
   delete(gameCamera);
}

/********************************************************************
 *                           renderGUI                              *
 ********************************************************************/
void xut::renderGUI()
{
   glDisable(GL_DEPTH_TEST);
   glDisable(GL_LIGHTING);
   glDisable(GL_FOG);
   /* Render the GUI elements */
   draw2DMode();
      
      /* Draw the Score */
      gameScore.draw();

      /* Draw the Status */
      if(rulesSystem)
      {
         gameStatus.setGlobalTouches(rulesSystem->getRemainingTouches());
         gameStatus.setActiveTeam(rulesSystem->getActiveTeam()->getTeamLogo());
         gameStatus.draw();
      }

      /* Draw the Messages */
      gameMessage.draw();

      /* Draw the main gui */
      if(state == XUT_STATE_NORMAL)
      {
         gameMainGui->render(mouseX, mouseY);
      }

      /* Draw the Mouse Cursor */
      glPushMatrix();
         cursors->draw(mouseX, mouseY);
      glPopMatrix();

   draw3DMode(FARVIEW);
   glEnable(GL_LIGHTING);
   glEnable(GL_FOG);
   glEnable(GL_DEPTH_TEST);
}

/********************************************************************
 *                            render                                *
 ********************************************************************/
void xut::render()
{
   /* Clear Things */
   glClearColor(0.1,0.1,0.8,1.0);
   glClear (GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT | GL_STENCIL_BUFFER_BIT);
   glLoadIdentity();

   /* Redefine camera position */
   gameCamera->lookAt();

   /* Define the listener position */
   snd.setListenerPosition(gameCamera->getCameraX(), gameCamera->getCameraY(),
                           gameCamera->getCameraZ(), gameCamera->getTheta(),
                           gameCamera->getPhi(), gameCamera->getD(), 0.0f);

   /* Render the stadium */
   gameField->render();

   /* Render Ball */
   gameBall->render();

   /* Render Teams */
   teamA->render();
   teamB->render();

   /* Draw Force */
   force.render();

   /* Render goals */
   goals->render(gameCamera);

}

/********************************************************************
 *                         prepareToShoot                           *
 ********************************************************************/
void xut::prepareToShoot()
{
   /* tell that will shoot */
   rulesSystem->prepareToShoot();
   /* Select the goal keeper to move */
   if(rulesSystem->getActiveTeam() == teamA)
   {
      selectedPlayer = teamB->getGoalKeeper();
   }
   else
   {
      selectedPlayer = teamA->getGoalKeeper();
   }
   /* Change camera to the target goal */
   gameCamera->push();
   gameCamera->updateCamera(selectedPlayer->getPosX(), 0.0f, 
                            selectedPlayer->getPosZ(), 0.0f);
   if(selectedPlayer->getPosX() > (FIELD_X / 2.0f))
   {
      /* Upper goal */
      gameCamera->setPhi(90);
   }
   else
   {
      gameCamera->setPhi(270);
   }

   /* Enter goalKeeper position mode */
   state = XUT_STATE_GOAL_KEEPER_POSITION;
}

/*********************************************************************
 *                         updateMouseFloorPos                       *
 *********************************************************************/
void xut::updateMouseFloorPos()
{
   GLfloat wx = mouseX,
           wy = screenHeight - mouseY;

   GLdouble cX=0, cY=0, cZ=0;
   GLdouble fX=0, fY=0, fZ=0;

   GLfloat t=0, vd=0;

   /* Get at camera */
   gluUnProject(wx, wy, 0, modl, proj, viewPort, &cX, &cY, &cZ);

   /* Get at far */
   gluUnProject(wx, wy, 0.9, modl, proj, viewPort, &fX, &fY, &fZ);

   /* Calculate */
   vd = fY - cY;
   if(vd == 0)
   {
      /* Vector Paralell to the floor */
      return;
   }
   t = (-cY / vd);
   xReal = cX + t*(fX-cX);
   zReal = cZ + t*(fZ-cZ);
}

/********************************************************************
 *                            gameIO                                *
 ********************************************************************/
void xut::gameIO()
{
   /* Update Matrix */
   glGetDoublev(GL_MODELVIEW_MATRIX, modl);
	glGetDoublev(GL_PROJECTION_MATRIX, proj);

   updateMouseFloorPos();

   if(state == XUT_STATE_NORMAL)
   {
      normalGameIO();
   }
   else if(state == XUT_STATE_GOAL_KEEPER_POSITION)
   {
      goalKeeperPositionIO();
   }
   else if(state == XUT_STATE_DISK_POSITION)
   {
      diskPositionIO();
   }
}

/********************************************************************
 *                        normalGameIO                              *
 ********************************************************************/
void xut::normalGameIO()
{
   team* activeTeam;

   int event = -1;

   if(selectedPlayer == NULL)
   {
      /* Verify Main GUI Actions */
      event = gameMainGui->verifyEvents(mButton, mouseX, mouseY);
      if(event != MAIN_GUI_NOTHING)
      {
         switch(event)
         {
            case MAIN_GUI_BUTTON_SHOOT:
               prepareToShoot(); 
            break;
            case MAIN_GUI_BUTTON_REPLAY:
               /* Enter replay mode */
               replayer->renderReplay(cursors);
            break;
         }
       
         /* Must return (done with verifications) */
         return;
      }


      /* Get Team to act */
      activeTeam = rulesSystem->getActiveTeam();

      if(activeTeam != NULL)
      {
         /* Try to get a player */
         if( mButton & SDL_BUTTON(1) )
         {
            selectedPlayer = activeTeam->selectPlayer(xReal, zReal);
            if(selectedPlayer)
            {
               if(rulesSystem->setDisk(selectedPlayer))
               {
                  gameCamera->setMoveWithMouse(false);
                  force.setInitial(selectedPlayer->getPosX(),
                                   selectedPlayer->getPosZ());
               }
               else
               {
                  /* Unselect it */
                  selectedPlayer = false;
                  gameMessage.set(gettext("This disk doesn't have more "
                                          "consecutive plays!"));
               }
            }
            else
            {
               /* Enable the Move the camera position */
               if(!gameCamera->getMoveWithMouse())
               {
                  // FIXME -> not working as desired.
                  // I know the error, but I'm too busy to fix it now.
                  //gameCamera->setMoveWithMouse(true);
               }
            }
         }
         else if( mButton & SDL_BUTTON(3) )
         {
            /* Show Disk Info */
            teamPlayer* tmpPlayer = activeTeam->selectPlayer(xReal, zReal);
            char buf[256];
            if(tmpPlayer)
            {
               sprintf(buf, gettext("Remaining Touches: %d"), 
                       rulesSystem->getRemainingTouches(tmpPlayer));
               gameMessage.set(buf);
            }
         }
      }
   }
   else
   {
      if(mButton & SDL_BUTTON(1) )
      {
         /* Is Doing Force Calculation */
         force.setFinal(xReal, zReal);
      }
      else
      {
         /* Done with force calculation. Do the Shoot! */
         float value=0, dX=0, dZ=0;
         force.getForce(value, dX, dZ);
         selectedPlayer->setVelocity(dX, dZ, value);
         force.clear();
         initedTurn = false;

         /* init a contact sound */
         snd.addSoundEffect(selectedPlayer->getPosX(), 0,
                            selectedPlayer->getPosZ(), -1, 
                            "sndfx/disk_shoot.ogg");

         /* Unselect the player */
         selectedPlayer = NULL;

         /* update replay data before goes to next position */
         replayer->updateData();
      }
   }
}

/*********************************************************************
 *                       goalKeeperPositionIO                        *
 *********************************************************************/
void xut::goalKeeperPositionIO()
{
   float posX=xReal;
   float posZ=zReal;

   float midX = FIELD_X / 2.0f;
   float midZ = FIELD_Z / 2.0f;

   if((!selectedPlayer) && (! (mButton & SDL_BUTTON(1))))
   {
      /* Done with press */
      state = XUT_STATE_NORMAL;
      /* Take to previous position the camera */
      gameCamera->pop();
      return;
   }

   if(selectedPlayer)
   {
      /* define the new Z position */
      if( (posZ >= midZ - FIELD_LITTLE_AREA_DELTA_Z) &&
          (posZ <= midZ + FIELD_LITTLE_AREA_DELTA_Z) )
      {
         selectedPlayer->setPosition(selectedPlayer->getPosX(), posZ);
      }

      /* verify if is up or down */
      if(selectedPlayer->getPosX() > midX)
      {
         /* Up */
         if( (posX <= FIELD_X) &&
             (posX >= FIELD_X - FIELD_LITTLE_AREA_DELTA_X) )
         {
            selectedPlayer->setPosition(posX, selectedPlayer->getPosZ());
         }
      }
      else
      {
         /* Down */
         if( (posX >= 0) &&
             (posX <= FIELD_LITTLE_AREA_DELTA_X) )
         {
            selectedPlayer->setPosition(posX, selectedPlayer->getPosZ());
         }
      }

      /* Changes to the orientation angle */
      if( (mButton & SDL_BUTTON(2)) || 
          (keys[opt.getKey(XUT_KEY_ROTATE_LEFT)]) )
      {
         selectedPlayer->setAngle(selectedPlayer->getAngle()+2);
      }
      else if( (mButton & SDL_BUTTON(3)) || 
               (keys[opt.getKey(XUT_KEY_ROTATE_RIGHT)]) )
      {
         selectedPlayer->setAngle(selectedPlayer->getAngle()-2);
      }

      if(mButton & SDL_BUTTON(1) )
      {
         /* Defined the position, must wait until mouse press end */
         selectedPlayer = NULL;
      }
   }

}

/*********************************************************************
 *                          diskPositionIO                           *
 *********************************************************************/
void xut::diskPositionIO()
{
   if( (!selectedPlayer) && (!(mButton & SDL_BUTTON(1)) ) )
   {
      /* Done with mouse press */
      state = XUT_STATE_NORMAL;
      return;
   }

   if(selectedPlayer)
   {
      float diskRadius = (teamA->diskMax[0] - teamA->diskMin[0]) / 2.0f;
      float ballRadius = (gameBall->max[0] - gameBall->min[0]) / 2.0f;
      float posX=xReal, posZ=zReal;
      float vecX = xReal - gameBall->getPosX(), 
            vecZ = zReal - gameBall->getPosZ();
      float dist = sqrt( vecX*vecX + vecZ*vecZ );
      
      normalize(vecX, vecZ);

      /* Verify if not too far away from the ball */
      if(dist > 2.0f*diskRadius)
      {
         posX = gameBall->getPosX() + 2.0f*diskRadius*vecX;
         posZ = gameBall->getPosZ() + 2.0f*diskRadius*vecZ;
      }
      /* Or not too near */
      else if(dist < ballRadius+diskRadius)
      {
         posX = gameBall->getPosX() + (ballRadius+diskRadius)*vecX;
         posZ = gameBall->getPosZ() + (ballRadius+diskRadius)*vecZ;
      }

      /* define the new position */
      selectedPlayer->setPosition(posX, posZ);

      if(mButton & SDL_BUTTON(1) )
      {
         /* Defined the position, must wait until mouse press end */
         selectedPlayer = NULL;
      }
   }
}

/*********************************************************************
 *                        verifyRulesResult                          *
 *********************************************************************/
void xut::verifyRulesResult()
{
   /* Verify Rules Result  */
   rulesSystem->ballAtFinalPosition();

   /* Always set the camera to the new position if
    * changed the ball owner */
   if( (rulesSystem->changedTeamToAct()) && 
       (opt.getCameraType() > XUT_OPTIONS_CAMERA_TYPE_STATIC) )
   {
      gameCamera->updateCamera(gameBall->getPosX(), 0.0f, 
                               gameBall->getPosZ(), 0.0f);
   }

   switch(rulesSystem->getState())
   {
      case RULE_STATE_MIDDLE:
      {
         /* A goal occurs, so call the goal sound effect */
         snd.addSoundEffect(-1, "sndfx/goal.ogg");
         /* And  the replay */
         replayer->renderReplay(cursors);
         /* Clear the replay data */
         replayer->clear();
         /* and reset the teams and put ball at middle */
         rulesSystem->setPositions();
         /* Set the camera to the new position */
         gameCamera->updateCamera(gameBall->getPosX(), 0.0f, 
                                  gameBall->getPosZ(), 0.0f);
         /* And the referee horn! */
         snd.addSoundEffect(-1, "sndfx/apito.ogg");
      }
      break;

      case RULE_STATE_CORNER_KICK:
      case RULE_STATE_THROW_IN:
      case RULE_STATE_GOAL_KICK:
      case RULE_STATE_FREE_KICK:
      case RULE_STATE_PENALTY_KICK:
      {
         /* the referee horn! */
         snd.addSoundEffect(-1, "sndfx/apito.ogg");
         
         /* Set The Position */
         rulesSystem->setPositions();

         /* Remove all contacts, isolating the ball */
         physic.removeContacts(true);

         /* Select disk to do the kick */
         selectedPlayer = rulesSystem->getActiveTeam()->getNearestPlayer(
                                      gameBall->getPosX(), gameBall->getPosZ());

         /* set state to the disk position adjust */
         state = XUT_STATE_DISK_POSITION;

         /* Set the camera to the new position */
         if(opt.getCameraType() > XUT_OPTIONS_CAMERA_TYPE_STATIC)
         {
            gameCamera->updateCamera(gameBall->getPosX(), 0.0f, 
                                     gameBall->getPosZ(), 0.0f);
         }
      }
      break;

      case RULE_STATE_NORMAL:
      default:
      {
         /* Just remove all contacts */
         physic.removeContacts(false);
      }
      break;
   }

   /* Init a new turn */
   initedTurn = true;
   rulesSystem->newTurn();

   /* Calculate angle from the ball to the desired goal */
   float angle = 0;

   if( ( (rulesSystem->getActiveTeam() == teamA) && 
         (rulesSystem->isFirstHalf()) ) ||
       ( (rulesSystem->getActiveTeam() == teamB) && 
         (!rulesSystem->isFirstHalf()) ) )
   {
      angle = 90;
   }
   else
   {
      angle = 270;
   }

   /* Set the new camera orientation */
   gameCamera->setPhi(angle);

   /* If the ball isn't at the visible culling, put the camera to it */
   //TODO
}

/*********************************************************************
 *                     redefine Window to Screen                     *
 *********************************************************************/
void xut::redefineWindow(SDL_Surface *screen, int actualFarView)
{
   glViewport (0, 0, screenWidth, screenHeight);
   glMatrixMode (GL_PROJECTION);
   glLoadIdentity ();
   gluPerspective(45.0, screenWidth / (float)screenHeight, 1.0, 
                  actualFarView);
   glGetIntegerv(GL_VIEWPORT, viewPort);
   glMatrixMode (GL_MODELVIEW);
   glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);

   glEnable(GL_LINE_SMOOTH);
   glHint(GL_LINE_SMOOTH_HINT, GL_DONT_CARE);
   glLineWidth (1.5);

   /* Details Definition */
   glDepthFunc(GL_LEQUAL);
   glEnable(GL_DEPTH_TEST);
   glShadeModel(GL_SMOOTH);

   /* Light Definition */
   GLfloat light_ambient[] = { 0.8, 0.8, 0.8, 1.0 };
   glLightModelfv(GL_LIGHT_MODEL_AMBIENT, light_ambient);
   glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_FALSE);
   glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);

   glEnable(GL_LIGHTING);
   //glEnable(GL_NORMALIZE);

      GLdouble fogEnd = HALFFARVIEW+200;
      GLdouble fogStart = 200;
      GLdouble fogDensity = 0.001;
      GLfloat color[3] = {0.8,0.8,0.8};
      glEnable(GL_FOG);
      {
        glFogi(GL_FOG_MODE,GL_LINEAR);
        glFogfv(GL_FOG_COLOR, color);
        glFogf(GL_FOG_DENSITY, fogDensity);
        glHint(GL_FOG_HINT, GL_DONT_CARE);
        glFogf(GL_FOG_START, fogStart);
        glFogf(GL_FOG_END, fogEnd);
      }
   glEnable(GL_MULTISAMPLE);

   /* Update Matrix */
   glGetDoublev(GL_MODELVIEW_MATRIX, modl);
	glGetDoublev(GL_PROJECTION_MATRIX, proj);
}

/********************************************************************
 *                      loadBackgroundTexture                       *
 ********************************************************************/
void xut::loadBackgroundTexture()
{
   dirs dir;
   glGenTextures(1, &bgTexture);
   SDL_Surface* img = IMG_Load(dir.getRealFile("gui/fundo.png").c_str());
   if(img)
   {
      setTextureRGBA(img, bgTexture);
      SDL_FreeSurface(img);
   }
   else
   {
      printf("Can't load Background texture!\n");
   }
}

/*********************************************************************
 *                             SplashScreen                          *
 *********************************************************************/
void xut::splashScreen()
{
   GLuint id;
   dirs dir;
   bool done = false;
   Uint32 t = SDL_GetTicks();

   SDL_Surface* img;
   img = IMG_Load(dir.getRealFile("gui/inicio1.png").c_str()); 
   glDisable(GL_LIGHTING);
   glDisable(GL_FOG);

   draw2DMode();

   dntFont fnt;
   fnt.defineFont(DNT_FONT_TIMES, 20);
   fnt.defineFontAlign(DNT_FONT_ALIGN_CENTER);
   color_Set(48,3,3, 255);
   fnt.write(img, 150, 370, gettext("Xut is developed under GPL license: "
                                   "Feel free to use its source code "
                                   "and data the way you want (or to blame us "
                                   "the same way). Xut is thankful "
                                   "to the people who developed those "
                                   "libraries: OpenAL, Cal3D, OpenGL, "
                                   "SDL, SDL_image, SDL_ttf, libVorbis."), 
              150, 370, 659, 500);
   fnt.defineFontAlign(DNT_FONT_ALIGN_LEFT);
   glGenTextures(1,&id);
   setTextureRGBA(img,id);
   SDL_FreeSurface(img);

   /* Fade In Screen */
   fadeInTexture(id,0,0,opt.getWindowX(),opt.getWindowY(), 800, 600);

   /* Wait until Mouse Button pressed or time passed */
   while( !done )
   {
      SDL_PumpEvents();
      mButton = SDL_GetMouseState(&mouseX,&mouseY);
      keys = SDL_GetKeyState(NULL);

      done = ( (mButton & SDL_BUTTON(1)) || 
               (keys[SDLK_ESCAPE]) ||
               (keys[SDLK_RETURN]) ||
               ( (SDL_GetTicks() - t) > 5000) );

      SDL_Delay(50);
   }

   /* Wait Mouse Button release */
   while(mButton & SDL_BUTTON(1))
   {
      SDL_PumpEvents();
      mButton = SDL_GetMouseState(&mouseX,&mouseY);
      SDL_Delay(50);
   }

   fadeOutTexture(id,0,0,opt.getWindowX(),opt.getWindowY(), 800, 600);
   
   glDeleteTextures(1,&id);
   
   draw3DMode(FARVIEW);
}

/*********************************************************************
 *                            fadeInTexture                          *
 *********************************************************************/
void xut::fadeInTexture(GLuint id, int x1, int y1, int x2, int y2,
                        int sizeX, int sizeY)
{
   int i;
   
   for(i=0; i < 50; i++)
   {
      glClearColor(0.1,0.1,0.8,1.0);
      glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT | 
              GL_STENCIL_BUFFER_BIT);
      glLoadIdentity();
      glColor3f(i/50.0, i/50.0, i/50.0);
      textureToScreen(id,x1,y1,x2,y2,sizeX,sizeY);
      glFlush();
      SDL_GL_SwapBuffers();
      SDL_Delay(10);
   }
   glColor4f(1.0,1.0,1.0, 1.0);
}

/*********************************************************************
 *                            fadeOutTexture                         *
 *********************************************************************/
void xut::fadeOutTexture(GLuint id, int x1, int y1, int x2, int y2,
                         int sizeX, int sizeY)
{
   int i;
   for(i=49; i >= 0; i--)
   {
      glClearColor(0.1,0.1,0.8,1.0);
      glClear(GL_DEPTH_BUFFER_BIT | GL_COLOR_BUFFER_BIT | 
              GL_STENCIL_BUFFER_BIT);
      glLoadIdentity();
      glColor3f(i/50.0, i/50.0, i/50.0);
      textureToScreen(id,x1,y1,x2,y2,sizeX,sizeY);
      glFlush();
      SDL_GL_SwapBuffers();
      SDL_Delay(10);
   }
   glColor4f(1.0, 1.0, 1.0, 1.0);
}

/********************************************************************
 *                            mainLoop                              *
 ********************************************************************/
void xut::mainLoop()
{
   Uint32 time = 0;
   int sleepTime = 0;
   Uint32 varTime = 0;
   Uint32 lastTime = 0;
   bool enableIO = false;

   /* Begin the half */
   rulesSystem->startHalf();
   gameMessage.set(gettext("The Match Begins!"));

   while(!exit)
   {
      time = SDL_GetTicks();
      varTime = time - lastTime;

      if( ((varTime)) >= UPDATE_RATE)
      {
         /* Update Clock */
         lastTime = time;
         if(rulesSystem->updateClock())
         {
            //TODO
            //TIME OVER!
         }


         /* Get Keys and Mouse State */
         srand(time);
         SDL_PumpEvents();
         keys = SDL_GetKeyState(NULL);
         mButton = SDL_GetMouseState(&mouseX,&mouseY);

         /* Do Camera IO */
         gameCamera->doIO(keys, mButton, mouseX, mouseY, DELTA_CAMERA);

         /* Only need to update the phisycs or replay at normal state */
         if(state == XUT_STATE_NORMAL)
         {
            /* Do the Physics! */
            if(!physic.updatePositions())
            {
               if(!initedTurn)
               {
                  verifyRulesResult();
               }
               enableIO = true;
            }
            else
            {
               /* update replay data */
               replayer->updateData();
               enableIO = false;

               /* Follow ball, if the camera is defined to it */
               if(opt.getCameraType() == XUT_OPTIONS_CAMERA_TYPE_FOLLOW_BALL)
               {
                  gameCamera->updateCamera(gameBall->getPosX(), 0.0f, 
                                           gameBall->getPosZ(), 0.0f);
               }
            }
         }

         /* Verify exit key */
         if(keys[SDLK_ESCAPE])
         {
            exit =true;
         }
         
         /* Verify replay key */
         if(state == XUT_STATE_NORMAL)
         {
            if(keys[opt.getKey(XUT_KEY_REPLAY)])
            {
               replayer->renderReplay(cursors);
            }
            else if(keys[opt.getKey(XUT_KEY_WILL_SHOOT)])
            {
               prepareToShoot();
            }
         }

         /* Do the game IO before draw the GUI to it not affect 
          * the gluUnProject */
         if(enableIO)
         {
            gameIO();
         }
         
         //TODO get goblin state

         /* Render */
         render();         

         renderGUI();

         /* Flush to Screen */
         glFlush();
         SDL_GL_SwapBuffers();

      }
      else
      {
         sleepTime = (int) ((UPDATE_RATE-1) - varTime);
         if( (sleepTime > 0) && (sleepTime < UPDATE_RATE))
         {
            SDL_Delay(sleepTime);
         }
      }

   }
}

/********************************************************************
 *                              run                                 *
 ********************************************************************/
void xut::run()
{
   int res = -1;
   
   /* Disable Default Mouse Cursor */
   SDL_ShowCursor(SDL_DISABLE);

   /* Put at 3D draw mode */
   redefineWindow(actualScreen, FARVIEW);

   /* Do The Splash Screen */
   splashScreen();

   initial* initialScreen = new initial(proj, modl, viewPort, bgTexture);


   while(res != INITIAL_EXIT)
   {
      exit = false;
      /* Run Initial Screen */
      res = initialScreen->run(cursors);

      switch(res)
      {
         case INITIAL_NEW:
         {
            selector* selTeam = new selector(bgTexture);
            if(selTeam->run(cursors) == SELECTOR_DONE)
            {
               /* Create TeamA */
               if(teamA)
               {
                  delete(teamA);
               }
               teamA = new team(selTeam->getTeamA());

               /* Create TeamB */
               if(teamB)
               {
                  delete(teamB);
               }
               teamB = new team(selTeam->getTeamB());

               /* Set the Score */
               gameScore.reset(teamA->getTeamLogo(), teamB->getTeamLogo());

               /* Tell Physics about Selected Teams */
               physic.setTeamA(teamA);
               physic.setTeamB(teamB);

               /* Tell Rules About Selected Teams */
               rulesSystem->clear();
               rulesSystem->setTeamA(teamA);
               rulesSystem->setTeamB(teamB);
               
               /* Tell Replayer About Selected Teams */
               replayer->setTeamA(teamA);
               replayer->setTeamB(teamB);
            
               /* Restarting replayer */
               replayer->clear();

               state = XUT_STATE_NORMAL;
               
               /* Free the Selector */
               delete(selTeam);
               
               /* Run the Main Loop */
               mainLoop();
            }
            else
            {
               /* Free the Selector */
               delete(selTeam);
            }
         }
         break;
         case INITIAL_OPTIONS:
         {
            /* Call the options screen */
            optScreen* optionsScreen = new optScreen(proj, modl, 
                                                     viewPort, bgTexture);
            optionsScreen->run(cursors);
            delete(optionsScreen);
         }
         break;
         case INITIAL_EXIT:
         {
            //TODO, some effect!
         }
         break;
      }
   }

   /* Put at 2D state */
   draw2DMode();

   /* Free the game score things */
   gameScore.quit();
   gameStatus.quit();
   gameMessage.quit();

   delete(initialScreen);

   /* Enable Default Cursor */
   SDL_ShowCursor(SDL_ENABLE);
}


