#include "collision.h"

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <iostream>
using namespace std;

/* TODO:
 * -physic for disk and goal collisions
 * -physic for ball and goal collisions
 */

/*****************************************************************
 *                           Constructor                         *
 *****************************************************************/
collision::collision()
{
   teamA = NULL;
   teamB = NULL;
}

/*****************************************************************
 *                           Destructor                          *
 *****************************************************************/
collision::~collision()
{
}

/*****************************************************************
 *                            setTeamA                           *
 *****************************************************************/
void collision::setTeamA(team* t)
{
   teamA = t;
}

/*****************************************************************
 *                            setTeamB                           *
 *****************************************************************/
void collision::setTeamB(team* t)
{
   teamB = t;
}

/*****************************************************************
 *                            setBall                            *
 *****************************************************************/
void collision::setBall(ball* b)
{
   gameBall = b;
}

/*****************************************************************
 *                          setTeamGoals                         *
 *****************************************************************/
void collision::setGoals(goal* g)
{
   goals = g;
}

/*****************************************************************
 *                         updatePositions                       *
 *****************************************************************/
bool collision::updatePositions()
{
   int i;

   bool moving = false;

   /* Update Team A Positions */
   if(teamA)
   {
      /* Update goal keeper */
      if(teamA->goalKeeper.updatePosition(FRICTION))
      {
         moving = true;
      }

      /* Update disks */
      for(i = 0; i < 10; i++)
      {
         if(teamA->disk[i].updatePosition(FRICTION))
         {
            moving = true;
         }
      }
   }

   /* Update Team B Positions */
   if(teamB)
   {
      /* Update goalkeeper */
      if(teamB->goalKeeper.updatePosition(FRICTION))
      {
         moving = true;
      }

      /* Update disks */
      for(i = 0; i < 10; i++)
      {
         if(teamB->disk[i].updatePosition(FRICTION))
         {
            moving = true;
         }
      }
   }

   /* Updating ball */
   if(gameBall->updatePosition())
   {
      moving = true;
   }

   verifyCollisions();
   
   return(moving);   
}

/*****************************************************************
 *                         verifyCollisions                      *
 *****************************************************************/
void collision::verifyCollisions()
{
   checkPlayerCollision();
   checkBallCollision();
}

/*****************************************************************
 *                         removeContacts                        *
 *****************************************************************/
void collision::removeContacts(bool isolateBall)
{
   int i;

   /* Verify Each Team */
   team* curTeam = teamA;
   while(curTeam)
   {
      /* Verify Each Disk */
      for(i=0; i < 10; i++)
      {
         if(removeContact(isolateBall, &curTeam->disk[i]))
         {
            /* One or more contacts were removed. Must verify all
             * disks again with the new defined position. */
            i = 0;
            curTeam = teamA;
         }
      }

      /* Verify GoalKeeper */
      removeContact(isolateBall, &curTeam->goalKeeper);

      /* Select next team or end */
      if(curTeam != teamB)
      {
         curTeam = teamB;
      }
      else
      {
         curTeam = NULL;
      }
   }
}

/*****************************************************************
 *                         removeContact                         *
 *****************************************************************/
bool collision::removeContact(bool isolateBall, teamPlayer* disk)
{
   float diskRadius = (teamA->diskMax[0] - teamA->diskMin[0]) / 2.0f;
   float dist;
   float midZ = FIELD_Z / 2.0f;

   /* Make sure the disk is in the field  */
   if(disk->getPosX()-diskRadius < FIELD_SIDE_DELTA)
   {
      disk->setPosition(diskRadius+FIELD_SIDE_DELTA, disk->getPosZ());
   }
   else if(disk->getPosX()+diskRadius > FIELD_X-FIELD_SIDE_DELTA)
   {
      disk->setPosition(FIELD_X-diskRadius-FIELD_SIDE_DELTA, disk->getPosZ());
   }
   if(disk->getPosZ()-diskRadius < FIELD_SIDE_DELTA)
   { 
      disk->setPosition(disk->getPosX(), diskRadius+FIELD_SIDE_DELTA);
   }
   else if(disk->getPosZ()+diskRadius > FIELD_Z-FIELD_SIDE_DELTA)
   {
      disk->setPosition(disk->getPosX(), FIELD_Z-diskRadius-FIELD_SIDE_DELTA);
   }

   /* Make sure disk isn't inner the extended small area */
   if((disk != &teamA->goalKeeper) && (disk != &teamB->goalKeeper))
   {
      if(isPointIn(disk->getPosX(), disk->getPosZ(),
               FIELD_SIDE_DELTA-diskRadius, 
               midZ - FIELD_LITTLE_AREA_DELTA_Z - FIELD_SIDE_DELTA -diskRadius,
               FIELD_LITTLE_AREA_DELTA_X + diskRadius, 
               midZ + FIELD_LITTLE_AREA_DELTA_Z + FIELD_SIDE_DELTA+diskRadius))
      {
         /* Is inner the lower goal area */
         getAwayFrom(disk, -10000, /* -10000 to avoid out of field */ 
               midZ - FIELD_LITTLE_AREA_DELTA_Z-FIELD_SIDE_DELTA-diskRadius,
               FIELD_LITTLE_AREA_DELTA_X + diskRadius,
               midZ + FIELD_LITTLE_AREA_DELTA_Z+FIELD_SIDE_DELTA+diskRadius);
         return(true);
      }
      else if(isPointIn(disk->getPosX(), disk->getPosZ(),
               FIELD_X - (FIELD_SIDE_DELTA + FIELD_LITTLE_AREA_DELTA_X + 
                  diskRadius), 
               midZ - FIELD_LITTLE_AREA_DELTA_Z - FIELD_SIDE_DELTA - 
               diskRadius,
               FIELD_X - FIELD_SIDE_DELTA - diskRadius,
               midZ + FIELD_LITTLE_AREA_DELTA_Z + FIELD_SIDE_DELTA + 
               diskRadius))
      {
         /* Is inner the upper goal area */
         getAwayFrom(disk,  
               FIELD_X - (FIELD_SIDE_DELTA + FIELD_LITTLE_AREA_DELTA_X 
                  + diskRadius),
               midZ - FIELD_LITTLE_AREA_DELTA_Z -FIELD_SIDE_DELTA-diskRadius,
               10000, /* 10000 to avoid out of field */
               midZ + FIELD_LITTLE_AREA_DELTA_Z + FIELD_SIDE_DELTA + 
               diskRadius);
         return(true);
      }
   }

   /* Verify contact with the ball */
   float ballRadius = (gameBall->max[0] - gameBall->min[0]) / 2.0f;
   if(isolateBall)
   {
      ballRadius += 3.0f*diskRadius;
   }
   dist = sqrt( (gameBall->getPosX() - disk->getPosX()) * 
                (gameBall->getPosX() - disk->getPosX()) +
                (gameBall->getPosZ() - disk->getPosZ()) * 
                (gameBall->getPosZ() - disk->getPosZ()) );
   if(dist <= ballRadius+diskRadius)
   { 
      /* Have a contact! Must get away from the ball */
      getAwayFrom(disk, gameBall->getPosX(), gameBall->getPosZ(),
                  ballRadius+diskRadius);
      return(true);
   }

   /* Verify contact with all other disks */
   team* curTeam = teamA;
   int i;
   while(curTeam != NULL)
   {
      for(i = 0; i < 10; i++)
      {
         /* Verify contact, if the disks are distinct */
         if(disk != &curTeam->disk[i])
         {
            dist =  sqrt( (curTeam->disk[i].getPosX() - disk->getPosX()) * 
                          (curTeam->disk[i].getPosX() - disk->getPosX()) +
                          (curTeam->disk[i].getPosZ() - disk->getPosZ()) * 
                          (curTeam->disk[i].getPosZ() - disk->getPosZ()) );
            if(dist <= 2*diskRadius)
            { 
               /* Have a contact! Must get away from the disk */
               getAwayFrom(disk, curTeam->disk[i].getPosX(), 
                           curTeam->disk[i].getPosZ(), 2*diskRadius);
               return(true);
            }
         }
      }

      /* Verify Contact with the goal and goalKeeper! */
      //TODO

      /* Look at the next team or end */
      if(curTeam == teamA)
      {
         curTeam = teamB;
      }
      else
      {
         curTeam = NULL;
      }
   }

   return(false);
}

/*****************************************************************
 *                          getAwayFrom                          *
 *****************************************************************/
void collision::getAwayFrom(teamPlayer* disk, float posX, 
                            float posZ, float radius)
{
   /* Radius+1 to be > not >= */
   radius += 1;

   /* Get the direction to the point */
   float vecX, vecZ;
   vecX = posX - disk->getPosX();
   vecZ = posZ - disk->getPosZ();
   normalize(vecX, vecZ);

   /* And Go away from that direction */
   disk->setPosition(posX-radius*vecX, posZ-radius*vecZ);
}

/*****************************************************************
 *                          getAwayFrom                          *
 *****************************************************************/
void collision::getAwayFrom(teamPlayer* disk, float x1, float z1, 
                       float x2, float z2)
{
   float d1, d2, d3, d4;

   d1 = disk->getPosX() - x1;
   d2 = disk->getPosZ() - z1;
   d3 = x2 - disk->getPosX();
   d4 = z2 - disk->getPosZ();

   if( (d1 <= d2) && (d1 <= d3) && (d1 <= d4) )
   {
      disk->setPosition(x1-1, disk->getPosZ());
   }
   else if( (d2 <= d3) && (d2 <= d4))
   {
      disk->setPosition(disk->getPosX(), z1-1);
   }
   else if(d3 <= d4)
   {
      disk->setPosition(x2+1, disk->getPosZ());
   }
   else
   {
      disk->setPosition(disk->getPosX(), z2+1);
   }
}

/*****************************************************************
 *                       checkPlayerCollision                    *
 *****************************************************************/
void collision::checkPlayerCollision()
{ 
   int i, j;
   float x, z;
   float minObj1[3];
   float maxObj1[3];
   float minObj2[3];
   float maxObj2[3];
   float minDist;
   team *t = teamA;
   
   /* tsa!! */
   for(int tsa=0; tsa<2; tsa++)
   {
      /* Checking collision with disks of the same team */
      if(t)
      {
         for (i=0; i<10; i++)
         {
            /* Getting bounding box of 'i' disk */
            minObj1[0] = t->diskMin[0] + t->disk[i].getPosX();
            minObj1[1] = t->diskMin[1];
            minObj1[2] = t->diskMin[2] + t->disk[i].getPosZ();
            
            maxObj1[0] = t->diskMax[0] + t->disk[i].getPosX();
            maxObj1[1] = t->diskMax[1];
            maxObj1[2] = t->diskMax[2] + t->disk[i].getPosZ();
            
            /* Cheking collision with disk players */
            for (j=i+1; j<10; j++)
            {
               /* Getting bounding box of 'j' disk */
               minObj2[0] = t->diskMin[0] + t->disk[j].getPosX();
               minObj2[1] = t->diskMin[1];
               minObj2[2] = t->diskMin[2] + t->disk[j].getPosZ();
               
               maxObj2[0] = t->diskMax[0] + t->disk[j].getPosX();
               maxObj2[1] = t->diskMax[1];
               maxObj2[2] = t->diskMax[2] + t->disk[j].getPosZ();
               
               /* Checking bounding box interception */
               if(intercepts(minObj1, maxObj1, minObj2, maxObj2, 1))
               {
                  /* Verifying depth colision */
                  minDist = t->diskMax[0]*2;
                  if(diskDepthCollision(t->disk[i], t->disk[j], minDist))
                  {
                     /* Applying collision physics */
                     disk_diskCollision(t->disk[i], t, t->disk[j], t);
                     /* Telling the rule system what happaned */
                     x = (t->disk[i].getPosX() + 
                          t->disk[j].getPosX()) / 2.0;
                     z = (t->disk[i].getPosZ() + 
                          t->disk[j].getPosZ()) / 2.0;
                     rule.diskCollideDisk(t, t, x, z);
                     /* Collision Sound */
                     snd.addSoundEffect(x, 0, z, -1, "sndfx/disk_col.ogg");
                  }
               }
            }
            
            /* Checking collision with goalkeeper */
            if(checkDiskGoalkeeperCollision(t, t->disk[i], minObj1, maxObj1))
            {
               /* Applying collision physics */
               disk_goalkeeperCollision(t->goalKeeper, t, t->disk[i], t);
               /* Telling the rule system what happaned */
               x = (t->disk[i].getPosX() + 
                    t->goalKeeper.getPosX()) / 2.0;
               z = (t->disk[i].getPosZ() + 
                    t->goalKeeper.getPosZ()) / 2.0;
               rule.diskCollideDisk(t, t, x, z);
            }
            
            /* Checking collision with goals */
            checkDiskGoalCollision(t->disk[i], t, minObj1, maxObj1);

         }
      }
      t = teamB;
   }
   
   /* Check collisions between disks of different teams */
   if(teamA && teamB)
   {
      /*min. distance between two disks without collision*/
      minDist = teamA->diskMax[0] + teamB->diskMax[0];
      
      for(i=0; i<10; i++)
      {
         /* Getting bounding box of 'i' disk (teamA) */
         minObj1[0] = teamA->diskMin[0] + teamA->disk[i].getPosX();
         minObj1[1] = teamA->diskMin[1];
         minObj1[2] = teamA->diskMin[2] + teamA->disk[i].getPosZ();
         
         maxObj1[0] = teamA->diskMax[0] + teamA->disk[i].getPosX();
         maxObj1[1] = teamA->diskMax[1];
         maxObj1[2] = teamA->diskMax[2] + teamA->disk[i].getPosZ();
         
         /* Cheking collision with disk players */
         for(j=0; j<10; j++)
         {
            /* Getting bounding box of 'j' disk (teamB) */
            minObj2[0] = teamB->diskMin[0] + teamB->disk[j].getPosX();
            minObj2[1] = teamB->diskMin[1];
            minObj2[2] = teamB->diskMin[2] + teamB->disk[j].getPosZ();
            
            maxObj2[0] = teamB->diskMax[0] + teamB->disk[j].getPosX();
            maxObj2[1] = teamB->diskMax[1];
            maxObj2[2] = teamB->diskMax[2] + teamB->disk[j].getPosZ();

            /* Checking bounding box interception */
            if(intercepts(minObj1, maxObj1, minObj2, maxObj2, 1))
            {
               /* Verifying depth colision */
               if(diskDepthCollision(teamA->disk[i], teamB->disk[j], minDist))
               {
                  /* Applying collision physics */
                  disk_diskCollision(teamA->disk[i], teamA, 
                                     teamB->disk[j], teamB);
                  /* Telling the rule system what happaned */
                  x = (teamA->disk[i].getPosX() + 
                       teamB->disk[j].getPosX()) / 2.0;
                  z = (teamA->disk[i].getPosZ() + 
                       teamB->disk[j].getPosZ()) / 2.0;
                  rule.diskCollideDisk(teamA, teamB, x, z);
                  snd.addSoundEffect(x, 0, z, -1, "sndfx/disk_col.ogg");
                  //printf("buuum %d com %d\n",i,j);
               }
            }
         }
         
         /* Checking collision of teamA disk and teamB goalkeeper */
         if(checkDiskGoalkeeperCollision(teamB, teamA->disk[i], 
                                         minObj1, maxObj1))
         {
            /* Applying collision physics */
            disk_goalkeeperCollision(teamB->goalKeeper, teamB, 
                                     teamA->disk[i], teamA);            
            /* Telling the rule system what happaned */
            x = (teamA->disk[i].getPosX() + 
                 teamB->goalKeeper.getPosX()) / 2.0;
            z = (teamA->disk[i].getPosZ() + 
                 teamB->goalKeeper.getPosZ()) / 2.0;
            rule.diskCollideDisk(teamA, teamB, x, z);
         }
         
         /* Checking collision of teamB disk and teamA goalkeeper */
         minObj1[0] = teamB->diskMin[0] + teamB->disk[i].getPosX();
         minObj1[1] = teamB->diskMin[1];
         minObj1[2] = teamB->diskMin[2] + teamB->disk[i].getPosZ();
         
         maxObj1[0] = teamB->diskMax[0] + teamB->disk[i].getPosX();
         maxObj1[1] = teamB->diskMax[1];
         maxObj1[2] = teamB->diskMax[2] + teamB->disk[i].getPosZ();
         if(checkDiskGoalkeeperCollision(teamA, teamB->disk[i], 
                                         minObj1, maxObj1))
         {
            /* Applying collision physics */
            disk_goalkeeperCollision(teamA->goalKeeper, teamA, 
                                     teamB->disk[i], teamB);
            /* Telling the rule system what happaned */
            x = (teamB->disk[i].getPosX() + 
                 teamA->goalKeeper.getPosX()) / 2.0;
            z = (teamB->disk[i].getPosZ() + 
                 teamA->goalKeeper.getPosZ()) / 2.0;
            rule.diskCollideDisk(teamA, teamB, x, z);
         }
      }
   }
}


/*****************************************************************
 *                   checkDiskGoalkeeperCollision                *
 *****************************************************************/
bool collision::checkDiskGoalkeeperCollision(team *t, teamPlayer disk,
                                             float minObj2[3], 
                                             float maxObj2[3])
{
   float minObj1[3], maxObj1[3];
   float X[4];
   float Z[4];
   X[0] = t->goalKeeperMin[0];   Z[0] = t->goalKeeperMin[2];
   X[1] = t->goalKeeperMin[0];   Z[1] = t->goalKeeperMax[2];
   X[2] = t->goalKeeperMax[0];   Z[2] = t->goalKeeperMax[2];
   X[3] = t->goalKeeperMax[0];   Z[3] = t->goalKeeperMin[2];
   
   rotTransBoundingBox(t->goalKeeper.getAngle(), X, Z, 
                       t->goalKeeper.getPosX(), 0, 10, 
                       t->goalKeeper.getPosZ(), minObj1, maxObj1);

   if(intercepts(minObj1, maxObj1, minObj2, maxObj2, 0))
   {
      float radius = (maxObj2[0] - minObj2[0])/2.0;
      if(goalKeeperDepthCollision(t, disk.getPosX(), disk.getPosZ(), radius))
      {
         return true;
      }
   }
   
   return false;
}


/*****************************************************************
 *                     checkDiskGoalCollision                    *
 *****************************************************************/
void collision::checkDiskGoalCollision(teamPlayer &disk, team *t,
                                       float minObj2[3], float maxObj2[3])
{
   float minObj1[3], maxObj1[3];
   float X[4];
   float Z[4];
   X[0] = goals->min[0];   Z[0] = goals->min[2];
   X[1] = goals->min[0];   Z[1] = goals->max[2];
   X[2] = goals->max[0];   Z[2] = goals->max[2];
   X[3] = goals->max[0];   Z[3] = goals->min[2];
   
   /* upper goal */
   rotTransBoundingBox(90, X, Z, 
                       GOAL1_X, 0, goals->max[1], 
                       GOAL_Z, minObj1, maxObj1);

   if(intercepts(minObj1, maxObj1, minObj2, maxObj2, 0))
   {
      if(goals->model->depthCollision(90, GOAL1_X, 0, GOAL_Z,
                     minObj2, maxObj2))
      {
         disk_goalCollision(disk, t, true, minObj1, maxObj1);
      }
   }
   
   /* bottom goal */
   rotTransBoundingBox(-90, X, Z, 
                       GOAL2_X, goals->min[1], goals->max[1], 
                       GOAL_Z, minObj1, maxObj1);

   if(intercepts(minObj1, maxObj1, minObj2, maxObj2, 0))
   {
      if(goals->model->depthCollision(-90, GOAL2_X, 0, GOAL_Z,
                     minObj2, maxObj2))
      {
         disk_goalCollision(disk, t, false, minObj1, maxObj1);
      }
   }   
}


/*****************************************************************
 *                       checkBallCollision                      *
 *****************************************************************/
void collision::checkBallCollision()
{
   if(!gameBall) 
   {
      return;
   }
 
   bool done;
   int t,times;               /* How many dt will need to test */
   GLfloat incX, incY, incZ;  /* To inc ball positions test*/
   GLfloat pX=0, pY=0, pZ=0;  /* Ball Previous position */
   GLfloat dX, dY, dZ;        /* Position deltas */

   /* Calculate how many time we'll have to verify the collision,
    * trying to avoid bullet effects. */
   gameBall->getPreviousPosition(pX, pY, pZ);
   dX = fabs(gameBall->getPosX() -  pX);
   dY = fabs(gameBall->getPosY() -  pY);
   dZ = fabs(gameBall->getPosZ() -  pZ);
   if( (dX >= dY) && (dX >= dZ))
   {
      /* Use dX as base */
      times = (int)floor(dX);
   }
   else if( (dY >= dX) && (dY >= dZ))
   {
      /* Use dY as base */
      times = (int)floor(dY);
   }
   else 
   {
      /* Use dZ as base */
      times = (int)floor(dZ);
   }

   /* Make sure we verify at last two positions
    * (to avoid going inner the target, and latter outter the target
    *  what will result in a no-collision) */
   if(times < 2)
   {
      times = 2;
   }

   /* Calculate the incs */
   incX = dX / times;
   incY = dY / times;
   incZ = dZ / times;

   /* Do the verify */
   done = false;
   for(t = 0; ((t < times) && (!done)); t++)
   {
      /* The last check is at the current position! */
      if(t == times-1)
      {
         pX = gameBall->getPosX();
         pY = gameBall->getPosY();
         pZ = gameBall->getPosZ();
      }
      else
      {
         pX += incX;
         pY += incY;
         pZ += incZ;
      }

      /* Do Ball Checks */
      done |= checkBallTeamCollision(teamA, pX, pY, pZ);
      done |= checkBallTeamCollision(teamB, pX, pY, pZ);
      done |= checkBallGoalCollision(pX, pY, pZ);

      /* Put the ball goBack to the previous checked position,
       * not the previous position, to make a better look collision effect */
      if(!done)
      {
         gameBall->setPreviousPosition(pX, pY, pZ);
      }
   }   
 
   /* Verify if some goal occurs */
   verifyGoals();

   /* Verify ball inner the field */
   checkBallFieldCollision();
}

/*****************************************************************
 *                            isInner                            *
 *****************************************************************/
bool collision::isInner(float min[3], float max[3], float x, float y, float z)
{
   return( ( (x >= min[0]) && (x <= max[0]) ) &&
           ( (y >= min[1]) && (y <= max[1]) ) &&
           ( (z >= min[2]) && (z <= max[2]) ) );
}

/*****************************************************************
 *                          verifyGoals                          *
 *****************************************************************/
void collision::verifyGoals()
{
   float halfX = FIELD_X / 2.0;
   bool upper = false;
   float X[4];
   float Z[4];
   float minGoal[3], maxGoal[3];

   /* Goal Bounding Box */
   X[0] = goals->min[0];   Z[0] = goals->min[2];
   X[1] = goals->min[0];   Z[1] = goals->max[2];
   X[2] = goals->max[0];   Z[2] = goals->max[2];
   X[3] = goals->max[0];   Z[3] = goals->min[2];

   /* Defint which goal the probally can be */
   upper = (gameBall->getPosX() > halfX);

   /* At Bottom Goal */
   if(!upper)
   {
      rotTransBoundingBox(-90, X, Z, GOAL2_X, 0, goals->max[1], 
                          GOAL_Z, minGoal, maxGoal);      
   }
   /* At Upper Goal */
   else
   {
      rotTransBoundingBox(90, X, Z, GOAL1_X, 0, goals->max[1], 
                          GOAL_Z, minGoal, maxGoal);
   }

   /* If ball inner goal, we got a goal! */
   if(isInner(minGoal, maxGoal, gameBall->getPosX(), gameBall->getPosY(),
            gameBall->getPosZ()))
   {
      rule.ballEnterGoal(upper);
   }
}


/*****************************************************************
 *                     checkBallTeamCollision                    *
 *****************************************************************/
bool collision::checkBallTeamCollision(team *t, float ballPosX, 
                                       float ballPosY, float ballPosZ)
{
   bool collisionGot = false;

   if (t == NULL)
   {
      return(false);
   }
   
   float minObj1[3];
   float maxObj1[3];
   float minObj2[3];
   float maxObj2[3];
   float minDist;
   
   /* Getting bounding box of ball */
   minObj1[0] = gameBall->min[0] + ballPosX;
   minObj1[1] = gameBall->min[1] + ballPosY + 2;
   minObj1[2] = gameBall->min[2] + ballPosZ;
   
   maxObj1[0] = gameBall->max[0] + ballPosX;
   maxObj1[1] = gameBall->max[1] + ballPosY + 2;
   maxObj1[2] = gameBall->max[2] + ballPosZ;

   /* checking ball collision with disk players */
   for (int i=0; i<10; i++)
   {
      /* Getting bounding box of 'i' disk */
      minObj2[0] = t->diskMin[0] + t->disk[i].getPosX();
      minObj2[1] = 0;
      minObj2[2] = t->diskMin[2] + t->disk[i].getPosZ();
      
      maxObj2[0] = t->diskMax[0] + t->disk[i].getPosX();
      maxObj2[1] = t->diskMax[1];
      maxObj2[2] = t->diskMax[2] + t->disk[i].getPosZ();
      
      /* Checking bounding box interception */
      if(intercepts(minObj1, maxObj1, minObj2, maxObj2, 0))
      {
         /* Verifying depth collision */
         minDist = t->diskMax[0] + gameBall->max[0];
         if(ballDepthCollision(t->disk[i], minDist))
         {
            /* applying collision force */
            disk_ballCollision(t->disk[i], t, gameBall);
            /* telling the rule system what happened */
            rule.ballCollideDisk(t);

            collisionGot = true;
         }
      }
   }
   
   /* checking ball collision with team goalkeeper */
   float X[4];
   float Z[4];
   X[0] = t->goalKeeperMin[0];   Z[0] = t->goalKeeperMin[2];
   X[1] = t->goalKeeperMin[0];   Z[1] = t->goalKeeperMax[2];
   X[2] = t->goalKeeperMax[0];   Z[2] = t->goalKeeperMax[2];
   X[3] = t->goalKeeperMax[0];   Z[3] = t->goalKeeperMin[2];
   
   rotTransBoundingBox(t->goalKeeper.getAngle(), X, Z, 
                       t->goalKeeper.getPosX(), t->goalKeeperMin[1], 
                       t->goalKeeperMax[1], 
                       t->goalKeeper.getPosZ(), minObj2, maxObj2);

   /* Checking bounding box interception */
   if(intercepts(minObj1, maxObj1, minObj2, maxObj2, 0))
   {
      /* Verifying depth collision in 2D (top vision) */
      if(t->goalKeeperModel->depthCollision(t->goalKeeper.getAngle(), 
                                            t->goalKeeper.getPosX(),
                                            0.0, t->goalKeeper.getPosZ(),
                                            minObj1, maxObj1))

      //if(goalKeeperDepthCollision(t, gameBall->getPosX(), 
      //                         gameBall->getPosZ(), gameBall->max[0]))
      {
         /* applying collision force */
         ball_goalkeeperCollision(t->goalKeeper, t, gameBall);
         /* telling the rule system what happened */
         rule.ballCollideDisk(t);

         collisionGot = true;
         //printf(" diiiiida!\n");
      }
   }

   return(collisionGot);
}


/*****************************************************************
 *                     checkBallGoalCollision                    *
 *****************************************************************/
bool collision::checkBallGoalCollision(float pX, float pY, float pZ)
{
   bool collisionGot = false;
   float halfX = FIELD_X / 2.0;
   float minObj1[3], maxObj1[3], minObj2[3], maxObj2[3];
   float X[4];
   float Z[4];
   X[0] = goals->min[0];   Z[0] = goals->min[2];
   X[1] = goals->min[0];   Z[1] = goals->max[2];
   X[2] = goals->max[0];   Z[2] = goals->max[2];
   X[3] = goals->max[0];   Z[3] = goals->min[2];
   
   /* Getting position and bounding box of ball */
   pY += 2;
   
   minObj1[0] = gameBall->min[0] + pX;
   minObj1[1] = gameBall->min[1] + pY;
   minObj1[2] = gameBall->min[2] + pZ;
   
   maxObj1[0] = gameBall->max[0] + pX;
   maxObj1[1] = gameBall->max[1] + pY;
   maxObj1[2] = gameBall->max[2] + pZ;
  
   /* Setting the goal Bouding box  */
   if(pX > halfX)
   {
      /* Getting bounding box of upper goal */
      rotTransBoundingBox(90, X, Z, GOAL1_X, 0, goals->max[1], 
                          GOAL_Z, minObj2, maxObj2);
   }
   else
   {
      /* Getting bounding box of bottom goal */
      rotTransBoundingBox(-90, X, Z, GOAL2_X, 0, goals->max[1], 
                          GOAL_Z, minObj2, maxObj2);
   }

   /* Checking low level collision */
   if(intercepts(minObj1, maxObj1, minObj2, maxObj2, 0))
   {
      /* Checking depth collision */
      if(pX > halfX)
      {
         if(goals->model->depthCollision(90, GOAL1_X, 0, GOAL_Z,
                             minObj1, maxObj1))
         {
            ball_goalCollision(gameBall, true, minObj2, maxObj2);
            collisionGot = true;
         }
      }
      else
      {
         if(goals->model->depthCollision(-90, GOAL2_X, 0, GOAL_Z,
                             minObj1, maxObj1))
         {
            ball_goalCollision(gameBall, false, minObj2, maxObj2);
            collisionGot = true;
         }
      }
      /* expanding the ball BBox to check if it's inside the goal */
      /*else 
      {
         //maxObj1[0] = FIELD_X;
         goals->model->depthCollision(90, GOAL1_X, 0, GOAL_Z,
                          minObj1, maxObj1);
         collisionGot = true;
      }*/
   }
   
   return(collisionGot);
}


/*****************************************************************
 *                    checkBallFieldCollision                    *
 *****************************************************************/
void collision::checkBallFieldCollision()
{
   float px = gameBall->getPosX();
   float pz = gameBall->getPosZ();
   
   /* cheking if ball has exited through the field byline */
   if (px >= FIELD_X - FIELD_SIDE_DELTA)
   {
      rule.ballExitAtByline(true, pz);
      //printf("upper cornerkick\n");
   }
   else if (px <= FIELD_SIDE_DELTA)
   {
      rule.ballExitAtByline(false, pz);
      //printf("bottom cornerkick\n");
   }
   
   /* checking if ball has exited through the field sides */
   if (pz >= FIELD_Z - FIELD_SIDE_DELTA)
   {
      rule.ballExitAtSide(px, FIELD_Z - FIELD_SIDE_DELTA);
      //printf("sidekick\n");
   }
   else if (pz <= FIELD_SIDE_DELTA)
   {
      rule.ballExitAtSide(px, FIELD_SIDE_DELTA);
      //printf("sidekick\n");
   }   
}


/*****************************************************************
 *                       diskDepthCollision                      *
 *****************************************************************/
bool collision::diskDepthCollision(teamPlayer d1, teamPlayer d2, 
                                   float minDist)
{
   float dx, dz, dist;
   dx = d1.getPosX() - d2.getPosX();
   dz = d1.getPosZ() - d2.getPosZ();
   
   dist = dx*dx + dz*dz;
   
   /*instead of squarerooting dist, just power the minDist*/
   return (dist <= minDist*minDist);
}


/*****************************************************************
 *                       ballDepthCollision                      *
 *****************************************************************/
bool collision::ballDepthCollision(teamPlayer d1, float minDist)
{
   float dx, dz, dist;
   dx = d1.getPosX() - gameBall->getPosX();
   dz = d1.getPosZ() - gameBall->getPosZ();
   
   dist = dx*dx + dz*dz;
   
   /*instead of squarerooting dist, just power the minDist*/
   return (dist <= minDist*minDist);
}


/*****************************************************************
 *                     goalKeeperDepthCollision                  *
 *****************************************************************/
bool collision::goalKeeperDepthCollision(team *t, float posX,
                                         float posZ, float radius)
{
   float angle;
   float x1, z1, Xt, Zt;
   
   /* getting distance to goalKeeper */
   x1 = posX - t->goalKeeper.getPosX();
   z1 = posZ - t->goalKeeper.getPosZ();
   
   /* rotating distance to goalkeeper coordinate system */
   angle = t->goalKeeper.getAngle();
   
   Xt = x1*cos(deg2Rad(-angle)) - z1*cos(PID2-deg2Rad(-angle));
   Zt = x1*sin(deg2Rad(-angle)) + z1*sin(PID2-deg2Rad(-angle));
   
   if (fabs(Xt) > radius + t->goalKeeperMax[0]) return false;
   if (fabs(Zt) > radius + t->goalKeeperMax[2]) return false;
   return true;
}

