#ifndef _collision_h
#define _collision_h

/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "../engine/team.h"
#include "../engine/ball.h"
#include "../engine/goal.h"
#include "../engine/rules.h"
#include "../engine/field.h"
#include "../etc/util.h"
#include "../sound/sound.h"
#include "physic.h"

/*! The collision class which verify fisic collisions
 * between the players, the ball and field elements. */
class collision
{
   public:
      /*! Constructor */
      collision();
      /*! Destructor */
      ~collision();

      /*! Define the Team A
       * \param t -> pointer to the teamA */
      void setTeamA(team* t);

      /*! Define the Team B
       * \param t -> pointer to the teamB */ 
      void setTeamB(team* t);
   
      /*! Define the ball
       * \param b -> pointer to the ball */
      void setBall(ball *b);
   
      /*! Define the goals
       * \param g -> pointer to the goals */
      void setGoals(goal *g);
      
      /*! Update all players and ball positions, also verifying the
       * collisions and telling rules system what happened
       * \return -> true if someone is moving, false if the
       *            physical system is stable. */
      bool updatePositions();
 
      /*! Remove all collisions from the system (re-ordering the players
       * on the field)
       * \param isolateBall -> if will put the ball isolated in a radius 
       *                       (usually used for fouls, etc.) */
      void removeContacts(bool isolateBall);

   protected:
      team* teamA;      /**< The teamA pointer */
      team* teamB;      /**< The teamB pointer */
      ball* gameBall;   /**< The ball used pointer */
      goal* goals;      /**< The goals pointer */
      rules rule;       /**< The ruleSystem */
      sound snd;        /**< The sound controller */

      /*! Remove All contacts of a disk.
       * \param isolateBall -> if will isolate the ball
       * \param disk -> pointer to the disk to remove contacts */
      bool removeContact(bool isolateBall, teamPlayer* disk);

      /*! Put the disk away from the position
       * \param disk -> disk to put away
       * \param posX -> X position
       * \param posZ -> Z position
       * \param radius -> radius to keep away from */
      void getAwayFrom(teamPlayer* disk, float posX, float posZ, 
                       float radius);
      
      /*! Put the disk away from the rectangular area
       * \param disk -> disk to put away
       * \param x1 -> lower x coordinate
       * \param z1 -> lower z coordinate
       * \param x2 -> upper x coordinate
       * \param z2 -> upper z coordinate */
      void getAwayFrom(teamPlayer* disk, float x1, float z1, 
                       float x2, float z2); 

      /*! Verify collisions at the world */
      void verifyCollisions();   
   
      /*! Verify colisions between players (disks, goalkepers) */
      void checkPlayerCollision();

      /*! Verify colision between disks and goalkeepers */
      bool checkDiskGoalkeeperCollision(team *t, teamPlayer disk,
                                        float minObj2[3], float maxObj2[3]);

      /*! Verify collision between disks and the goal */
      void checkDiskGoalCollision(teamPlayer &disk, team *t,
                                  float minObj2[3], float maxObj2[3]);
      
      /*! Verify all ball colisions */
      void checkBallCollision();

      bool isInner(float min[3], float max[3], float x, float y, float z);

      void verifyGoals();
      
      bool checkBallTeamCollision(team *t, float ballPosX, 
                                  float ballPosY, float ballPosZ);
      bool checkBallGoalCollision(float pX, float pY, float pZ);
      void checkBallFieldCollision();

      bool diskDepthCollision(teamPlayer d1, teamPlayer d2, float minDist);
      bool ballDepthCollision(teamPlayer d1, float minDist);
      bool goalKeeperDepthCollision(team *t, float posX, 
                                    float posZ, float radius);

};




#endif

