/*
  xut - button football (soccer) game
  Copyright (C) 2008 farrer <farrer@dnteam.org> and gushm <gushm@dnteam.org>

  This file is part of xut.

  xut is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  xut is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with xut.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "physic.h"

void disk_diskCollision(teamPlayer &d1, team* t1, 
                        teamPlayer &d2, team* t2)
{
   /* distance magnitude, vector and normalized vector */
   float dist, dX, dZ, dXu, dZu;

   /* disks velocity magnitude and vector in XZ coord. system */
   float v1, v1x, v1z;
   float v2, v2x, v2z;

   /* disks velocity vector in collision coord. system */
   /* vNp = velocity of disk N in collision direction before collision */
   /* vNn = velocity of disk N perpendicular to collision direction */
   /* vNpf = velocity of disk N in collision direction after collision */
   float v1n, v1p, v1pf;
   float v2n, v2p, v2pf; 

   /* elastic coeficient of collision */
   float elasticness = 0.5;

   /* masses for disk1 and disk2 */
   float mass1 = 1;
   float mass2 = 1;
   
   float minDist = t1->diskMax[0] + t2->diskMax[0];
  
   /* getting distance vector and magnitude */
   dX = d2.getPosX() - d1.getPosX();
   dZ = d2.getPosZ() - d1.getPosZ();
   dist = sqrt(dX*dX + dZ*dZ);

   if (dist < minDist){
      /* collision has happened some time ago, but we only realized now.
       * we must go back to the time approx before the collision happened. */

      d1.goBack();
      d2.goBack();
      
      dX = d2.getPosX() - d1.getPosX();
      dZ = d2.getPosZ() - d1.getPosZ();
      dist = sqrt(dX*dX + dZ*dZ);
   }

   if (dist == 0) return;
   
   dXu = dX/dist;
   dZu = dZ/dist;
   
   /* calculating initial velocities */
   v1 = d1.getVelocity();
   v1x = v1 * d1.getOrientX();
   v1z = v1 * d1.getOrientZ();
   
   v2 = d2.getVelocity();
   v2x = v2 * d2.getOrientX();
   v2z = v2 * d2.getOrientZ();
   
   /* calculating velocity components based on collision vector 
      coordinate system */
   v1p = v1x*dXu + v1z*dZu;
   v1n = (-v1x)*dZu + v1z*dXu;
   
   v2p = v2x*dXu + v2z*dZu;
   v2n = (-v2x)*dZu + v2z*dXu;
   
   /* calculating collision effects */
   v1pf = v1p + (1+elasticness)*(v2p-v1p)/(1+mass1/mass2);
   v2pf = v2p + (1+elasticness)*(v1p-v2p)/(1+mass2/mass1);
   
   /* transforming back into XZ coordinate system */
   v1x = v1pf*dXu - v1n*dZu;
   v1z = v1pf*dZu + v1n*dXu;
   
   v2x = v2pf*dXu - v2n*dZu;
   v2z = v2pf*dZu + v2n*dXu;
   
   /* getting orientation and magnitude for new velocities */
   v1 = sqrt(v1x*v1x + v1z*v1z);
   if (v1 != 0)
   {
      v1x /= v1;
      v1z /= v1;
   }
   else{
      v1x = 0;
      v1z = 0;
   }
   
   v2 = sqrt(v2x*v2x + v2z*v2z);
   if (v2 != 0)
   {
      v2x /= v2;
      v2z /= v2;
   }
   else{
      v2x = 0;
      v2z = 0;
   }
   
   /* applying changes */
   d1.setNextVelocity(v1x, v1z, v1);
   d2.setNextVelocity(v2x, v2z, v2);
}


void disk_ballCollision(teamPlayer &disk, team* t1, ball *gameBall)
{   
   /* distance magnitude, vector and normalized vector */
   float dist, dX, dZ, dXu, dZu;
   
   /* velocity magnitude and vector in XZ coord. system */
   float v1, v1x, v1z;
   float v2, v2x, v2z;
   
   /* velocity vectors in collision coord. system */
   /* vXp = velocity in collision direction before collision */
   /* vXn = velocity in perpendicular to collision direction */
   /* vXpf = velocity in collision direction after collision */
   float v1n, v1p, v1pf;
   float v2n, v2p, v2pf; 

   /* elastic coeficient of collision */
   float elasticness = 0.2;

   /* masses for ball and disk */
   float mass1 = 0.5;
   float mass2 = 1;
   
   /* collision has happened some time ago, but we only realized now.
    * we must go back to the time approx before the collision happened. */
   gameBall->goBack();
   
   /* checking if ball has collided with top or lateral of the disk
    * if its a top collision, we may just bounce the ball up, inverting its
    * velY signal */
    if (gameBall->getPosY() >= t1->diskMax[1])
    {
       gameBall->setPosY(t1->diskMax[1]+0.1);
       gameBall->setVelocityY((-0.4)*gameBall->getVelocityY());
       return;
    }
    
    disk.goBack();
   
   /* getting distance vector and magnitude */
   dX = disk.getPosX() - gameBall->getPosX();
   dZ = disk.getPosZ() - gameBall->getPosZ();
   dist = sqrt(dX*dX + dZ*dZ);
   
   if (dist == 0) return;
   
   dXu = dX/dist;
   dZu = dZ/dist;
   
   /* calculating initial velocities */
   v1 = gameBall->getVelocity();
   v1x = v1 * gameBall->getOrientX();
   v1z = v1 * gameBall->getOrientZ();
   
   v2 = disk.getVelocity();
   v2x = v2 * disk.getOrientX();
   v2z = v2 * disk.getOrientZ();
   
   /* calculating velocity components based on collision vector 
      coordinate system */
   v1p = v1x*dXu + v1z*dZu;
   v1n = (-v1x)*dZu + v1z*dXu;
   
   v2p = v2x*dXu + v2z*dZu;
   v2n = (-v2x)*dZu + v2z*dXu;
   
   /* calculating collision effects */
   v1pf = v1p + (1+elasticness)*(v2p-v1p)/(1+mass1/mass2);
   v2pf = v2p + (1+elasticness)*(v1p-v2p)/(1+mass2/mass1);
   
   /* transforming back into XZ coordinate system */
   v1x = v1pf*dXu - v1n*dZu;
   v1z = v1pf*dZu + v1n*dXu;
   
   v2x = v2pf*dXu - v2n*dZu;
   v2z = v2pf*dZu + v2n*dXu;
   
   /* getting orientation and magnitude for new velocities */
   v1 = sqrt(v1x*v1x + v1z*v1z);
   if (v1 != 0)
   {
      v1x /= v1;
      v1z /= v1;
   }
   else{
      v1x = 0;
      v1z = 0;
   }
   
   v2 = sqrt(v2x*v2x + v2z*v2z);
   if (v2 != 0)
   {
      v2x /= v2;
      v2z /= v2;
   }
   else{
      v2x = 0;
      v2z = 0;
   }
   
   /* applying changes */
   gameBall->setVelocity(v1x, v1z, v1);
   gameBall->setVelocityY(v1/4);
   disk.setNextVelocity(v2x, v2z, v2);
   
   /* updating positions back to current time */
   gameBall->updatePosition();
   disk.updatePosition(0);
}

void disk_goalkeeperCollision(teamPlayer &goalKeeper, team *goalTeam,
                              teamPlayer &disk, team *diskTeam)
{   
   /* goalKeeper angle*/
   float angle;
   
   /* disk distance to goalkeeper in XZ coordinate system*/
   float dX, dZ;
   
   /* disk distance to goalkeeper transformed to its coordinate system */
   float dXt, dZt;
   
   /* disk velocity in XZ coordinate system and magnitude*/
   float vBx, vBz, vB;
   
   /* disk velocity transformed to goalkeeper coordinate system */
   float vBxt, vBzt;
   
   /* collision has happened some time ago, but we only realized now.
    * we must go back to the time approx before the collision happened. */
   disk.goBack();
      
   /* getting goalkeeper angle */
   angle = goalKeeper.getAngle();
   
   /* getting the disk current distance to goalkeeper */
   dX = disk.getPosX() - goalKeeper.getPosX();
   dZ = disk.getPosZ() - goalKeeper.getPosZ();
   
   /* rotating the distance to the goalkeeper coordinate system */
   dXt = dX*cos(deg2Rad(-angle)) - dZ*cos(PID2-deg2Rad(-angle));
   dZt = dX*sin(deg2Rad(-angle)) + dZ*sin(PID2-deg2Rad(-angle));

   /* getting the disk current velocity */
   vBx = disk.getVelocity() * disk.getOrientX();
   vBz = disk.getVelocity() * disk.getOrientZ();   
   
   /* rotating the disk velocity to the goalkeeper coordinate system */
   vBxt = vBx*cos(deg2Rad(-angle)) - vBz*cos(PID2-deg2Rad(-angle));
   vBzt = vBx*sin(deg2Rad(-angle)) + vBz*sin(PID2-deg2Rad(-angle));
   
   /* calculating collision effects */
   /* if dXt is too low, disk may have collided with front of the goalkeeper */
   if (fabs(dXt) < goalTeam->goalKeeperMax[0])
   {
      vBxt = (0.6) * vBxt;
      vBzt = (-0.6) * vBzt;
   }
   /* else it collided with side of the goalkeeper */
   else{
      vBxt = (-0.6) * vBxt;
      vBzt = (0.6) * vBzt;
   }
   
   /* rotating velocity back to the XZ corrdinate system */
   vBx = vBxt*cos(deg2Rad(angle)) - vBzt*cos(PID2-deg2Rad(angle));
   vBz = vBxt*sin(deg2Rad(angle)) + vBzt*sin(PID2-deg2Rad(angle));
   
   /* getting velocity magnitude and normalizing vector */
   vB = sqrt(vBx*vBx + vBz*vBz);
   if (vB != 0)
   {
      vBx /= vB;
      vBz /= vB;
      
   }
   else{
      vBx = 0;
      vBz = 0;
   }
   
   /* applying changes */
   disk.setNextVelocity(vBx, vBz, vB);
   //disk.updatePosition(0);
}

void ball_goalkeeperCollision(teamPlayer &goalKeeper, team *t, ball *gameBall)
{   
   /* goalKeeper angle*/
   float angle;
   
   /* ball distance to goalkeeper in XZ coordinate system*/
   float dX, dZ;
   
   /* ball distance to goalkeeper transformed to its coordinate system */
   float dXt, dZt;
   
   /* ball velocity in XZ coordinate system and magnitude*/
   float vBx, vBz, vB;
   
   /* ball velocity transformed to goalkeeper coordinate system */
   float vBxt, vBzt;
   
   /* collision has happened some time ago, but we only realized now.
    * we must go back to the time approx before the collision happened. */
   gameBall->goBack();
      
   /* getting goalkeeper angle */
   angle = goalKeeper.getAngle();
   
   /* getting the current ball distance to goalkeeper */
   dX = gameBall->getPosX() - goalKeeper.getPosX();
   dZ = gameBall->getPosZ() - goalKeeper.getPosZ();
   
   /* rotating the distance to the goalkeeper coordinate system */
   dXt = dX*cos(deg2Rad(-angle)) - dZ*cos(PID2-deg2Rad(-angle));
   dZt = dX*sin(deg2Rad(-angle)) + dZ*sin(PID2-deg2Rad(-angle));

   /* getting the current ball velocity */
   vBx = gameBall->getVelocity() * gameBall->getOrientX();
   vBz = gameBall->getVelocity() * gameBall->getOrientZ();   
   
   /* rotating the ball velocity to the goalkeeper coordinate system */
   vBxt = vBx*cos(deg2Rad(-angle)) - vBz*cos(PID2-deg2Rad(-angle));
   vBzt = vBx*sin(deg2Rad(-angle)) + vBz*sin(PID2-deg2Rad(-angle));
   
   /* calculating collision effects */
   if (fabs(dZt) < t->goalKeeperMax[2])
   {
      vBxt = (-0.3) * vBxt;
      vBzt = 0.3 * vBzt;
   }
   else{
      vBxt = 0.3 * vBxt;
      vBzt = (-0.3) * vBzt;
   }
   
   /* rotating velocity back to the XZ corrdinate system */
   vBx = vBxt*cos(deg2Rad(angle)) - vBzt*cos(PID2-deg2Rad(angle));
   vBz = vBxt*sin(deg2Rad(angle)) + vBzt*sin(PID2-deg2Rad(angle));
   
   /* getting velocity magnitude and normalizing vector */
   vB = sqrt(vBx*vBx + vBz*vBz);
   if (vB != 0)
   {
      vBx /= vB;
      vBz /= vB;
      
   }
   else{
      vBx = 0;
      vBz = 0;
   }
   
   /* applying changes */
   gameBall->setVelocity(vBx, vBz, vB);
   
   /* updating ball position to current time */
   gameBall->updatePosition();

}

bool ball_goalCollision(ball *gameBall, bool upper, 
                        float minGoal[3], float maxGoal[3])
{
   /* ball current position */
   float ballX, ballY, ballZ;
   
   /* ball position before collision */
   float lastBx, lastBy, lastBz;
   
   /* booleans indicating where ball is crossing the goal bbox */
   bool crossX, crossY, crossZ;
   
   bool returnValue = false;
   
   /* getting ball positions */
   ballX = gameBall->getPosX();
   ballY = gameBall->getPosY() + 2;
   ballZ = gameBall->getPosZ();

   gameBall->goBack();
   
   lastBx = gameBall->getPosX();
   lastBy = gameBall->getPosY() + 2;
   lastBz = gameBall->getPosZ();
    
   /* checking ball position to guess where the collision occurred */
   if (upper)
   {
      crossX = (fabs(ballX - minGoal[0]) < 2);
   }
   else
   {
      crossX = (fabs(ballX - maxGoal[0]) < 2);
   }
   
   crossZ = (fabs(ballZ - minGoal[2]) < 2 || fabs(ballZ - maxGoal[2]) < 2);
   
   crossY = (fabs(ballY - maxGoal[1]) < 2) || (!crossX && !crossZ);
   
   /* if ball collided with goal side or top bars, invert its x orientation */
   if (crossX)
   {
      gameBall->setVelocity(-gameBall->getOrientX(), gameBall->getOrientZ(),
                            (0.3) * gameBall->getVelocity());
      returnValue = false;
   }
   
   /* if ball collided with goal side or back, invert its z orientation */
   else if (crossZ)
   {
      gameBall->setVelocity(gameBall->getOrientX(), -gameBall->getOrientZ(),
                            (0.3) * gameBall->getVelocity());
      /* if ball is inside the goal, tell it's a goal */
      if (ballZ > minGoal[2] && ballZ < maxGoal[2])
         returnValue = true;
      else 
         returnValue = false;
   }

   /* if collision system detected a collision and it wasnt in X or Z
    * limits of the goal, it must have been on goals back/top 
    * we must change the ball y speed along with the X orientation */
   /* we have a problem here. X orientation must be changed according
    * to the position of the ball. if ball hit the back/top from the inside
    * X orientation sign must be changed. but how can we know if ball is in or
    * outside the goal? */
   else 
   {
      gameBall->setVelocity(-gameBall->getOrientX(), gameBall->getOrientZ(),
                            (0.3) * gameBall->getVelocity());
      gameBall->setVelocityY((-0.3) * gameBall->getVelocityY());
      /* always return true in this case. the rule system checks if it's 
       * a goal or if ball is out of field */
      returnValue = true;
   }

   gameBall->updatePosition();
   return returnValue;
}



void disk_goalCollision(teamPlayer &disk, team *diskTeam, bool upper,
                        float minGoal[3], float maxGoal[3])
{
   /* disk current position */
   float posX, posZ;
   
   /* disk distance to the goal side */
   float distX;
   
   /* getting disk position */
   posX = disk.getPosX();
   posZ = disk.getPosZ();
   
   /* going back to the time before the collision happened */
   disk.goBack();
   
   /* checking disk distance to guess where the collision occurred */
   if (upper)
   {
      distX = minGoal[0] - posX;
   }
   else
   {
      distX = posX - maxGoal[0];
   }
   
   /* if distX > 0, disk is in front of the goal and has hit the sidebar */
   if (distX>0)
   {
      disk.setVelocity(-disk.getOrientX(), disk.getOrientZ(),
                      (0.3) * disk.getVelocity());
   }
   /* else disk is beside the goal and has hit its side */
   else
   {
      disk.setVelocity(disk.getOrientX(), -disk.getOrientZ(),
                      (0.3) * disk.getVelocity());
   }
   
   disk.updatePosition(0);
}




